# pylint: disable=too-many-lines,too-many-statements
# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
from io import IOBase
import sys
from typing import Any, Callable, Dict, IO, Iterable, Iterator, Optional, Type, TypeVar, Union, cast, overload
import urllib.parse

from azure.core.exceptions import (
    ClientAuthenticationError,
    HttpResponseError,
    ResourceExistsError,
    ResourceNotFoundError,
    ResourceNotModifiedError,
    StreamClosedError,
    StreamConsumedError,
    map_error,
)
from azure.core.paging import ItemPaged
from azure.core.pipeline import PipelineResponse
from azure.core.polling import LROPoller, NoPolling, PollingMethod
from azure.core.rest import HttpRequest, HttpResponse
from azure.core.tracing.decorator import distributed_trace
from azure.core.utils import case_insensitive_dict
from azure.mgmt.core.exceptions import ARMErrorFormat
from azure.mgmt.core.polling.arm_polling import ARMPolling

from .. import models as _models
from .._serialization import Serializer

if sys.version_info >= (3, 9):
    from collections.abc import MutableMapping
else:
    from typing import MutableMapping  # type: ignore  # pylint: disable=ungrouped-imports
T = TypeVar("T")
ClsType = Optional[Callable[[PipelineResponse[HttpRequest, HttpResponse], T, Dict[str, Any]], Any]]

_SERIALIZER = Serializer()
_SERIALIZER.client_side_validation = False


def build_list_build_services_request(
    resource_group_name: str, service_name: str, subscription_id: str, **kwargs: Any
) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    api_version: str = kwargs.pop("api_version", _params.pop("api-version", "2023-12-01"))
    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = kwargs.pop(
        "template_url",
        "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.AppPlatform/Spring/{serviceName}/buildServices",
    )  # pylint: disable=line-too-long
    path_format_arguments = {
        "subscriptionId": _SERIALIZER.url("subscription_id", subscription_id, "str"),
        "resourceGroupName": _SERIALIZER.url("resource_group_name", resource_group_name, "str"),
        "serviceName": _SERIALIZER.url("service_name", service_name, "str", pattern=r"^[a-z][a-z0-9-]*[a-z0-9]$"),
    }

    _url: str = _url.format(**path_format_arguments)  # type: ignore

    # Construct parameters
    _params["api-version"] = _SERIALIZER.query("api_version", api_version, "str")

    # Construct headers
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="GET", url=_url, params=_params, headers=_headers, **kwargs)


def build_get_build_service_request(
    resource_group_name: str, service_name: str, build_service_name: str, subscription_id: str, **kwargs: Any
) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    api_version: str = kwargs.pop("api_version", _params.pop("api-version", "2023-12-01"))
    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = kwargs.pop(
        "template_url",
        "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.AppPlatform/Spring/{serviceName}/buildServices/{buildServiceName}",
    )  # pylint: disable=line-too-long
    path_format_arguments = {
        "subscriptionId": _SERIALIZER.url("subscription_id", subscription_id, "str"),
        "resourceGroupName": _SERIALIZER.url("resource_group_name", resource_group_name, "str"),
        "serviceName": _SERIALIZER.url("service_name", service_name, "str", pattern=r"^[a-z][a-z0-9-]*[a-z0-9]$"),
        "buildServiceName": _SERIALIZER.url("build_service_name", build_service_name, "str"),
    }

    _url: str = _url.format(**path_format_arguments)  # type: ignore

    # Construct parameters
    _params["api-version"] = _SERIALIZER.query("api_version", api_version, "str")

    # Construct headers
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="GET", url=_url, params=_params, headers=_headers, **kwargs)


def build_create_or_update_request(
    resource_group_name: str, service_name: str, build_service_name: str, subscription_id: str, **kwargs: Any
) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    api_version: str = kwargs.pop("api_version", _params.pop("api-version", "2023-12-01"))
    content_type: Optional[str] = kwargs.pop("content_type", _headers.pop("Content-Type", None))
    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = kwargs.pop(
        "template_url",
        "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.AppPlatform/Spring/{serviceName}/buildServices/{buildServiceName}",
    )  # pylint: disable=line-too-long
    path_format_arguments = {
        "subscriptionId": _SERIALIZER.url("subscription_id", subscription_id, "str"),
        "resourceGroupName": _SERIALIZER.url("resource_group_name", resource_group_name, "str"),
        "serviceName": _SERIALIZER.url("service_name", service_name, "str", pattern=r"^[a-z][a-z0-9-]*[a-z0-9]$"),
        "buildServiceName": _SERIALIZER.url("build_service_name", build_service_name, "str"),
    }

    _url: str = _url.format(**path_format_arguments)  # type: ignore

    # Construct parameters
    _params["api-version"] = _SERIALIZER.query("api_version", api_version, "str")

    # Construct headers
    if content_type is not None:
        _headers["Content-Type"] = _SERIALIZER.header("content_type", content_type, "str")
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="PUT", url=_url, params=_params, headers=_headers, **kwargs)


def build_list_builds_request(
    resource_group_name: str, service_name: str, build_service_name: str, subscription_id: str, **kwargs: Any
) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    api_version: str = kwargs.pop("api_version", _params.pop("api-version", "2023-12-01"))
    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = kwargs.pop(
        "template_url",
        "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.AppPlatform/Spring/{serviceName}/buildServices/{buildServiceName}/builds",
    )  # pylint: disable=line-too-long
    path_format_arguments = {
        "subscriptionId": _SERIALIZER.url("subscription_id", subscription_id, "str"),
        "resourceGroupName": _SERIALIZER.url("resource_group_name", resource_group_name, "str"),
        "serviceName": _SERIALIZER.url("service_name", service_name, "str", pattern=r"^[a-z][a-z0-9-]*[a-z0-9]$"),
        "buildServiceName": _SERIALIZER.url("build_service_name", build_service_name, "str"),
    }

    _url: str = _url.format(**path_format_arguments)  # type: ignore

    # Construct parameters
    _params["api-version"] = _SERIALIZER.query("api_version", api_version, "str")

    # Construct headers
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="GET", url=_url, params=_params, headers=_headers, **kwargs)


def build_get_build_request(
    resource_group_name: str,
    service_name: str,
    build_service_name: str,
    build_name: str,
    subscription_id: str,
    **kwargs: Any
) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    api_version: str = kwargs.pop("api_version", _params.pop("api-version", "2023-12-01"))
    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = kwargs.pop(
        "template_url",
        "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.AppPlatform/Spring/{serviceName}/buildServices/{buildServiceName}/builds/{buildName}",
    )  # pylint: disable=line-too-long
    path_format_arguments = {
        "subscriptionId": _SERIALIZER.url("subscription_id", subscription_id, "str"),
        "resourceGroupName": _SERIALIZER.url("resource_group_name", resource_group_name, "str"),
        "serviceName": _SERIALIZER.url("service_name", service_name, "str", pattern=r"^[a-z][a-z0-9-]*[a-z0-9]$"),
        "buildServiceName": _SERIALIZER.url("build_service_name", build_service_name, "str"),
        "buildName": _SERIALIZER.url("build_name", build_name, "str"),
    }

    _url: str = _url.format(**path_format_arguments)  # type: ignore

    # Construct parameters
    _params["api-version"] = _SERIALIZER.query("api_version", api_version, "str")

    # Construct headers
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="GET", url=_url, params=_params, headers=_headers, **kwargs)


def build_create_or_update_build_request(
    resource_group_name: str,
    service_name: str,
    build_service_name: str,
    build_name: str,
    subscription_id: str,
    **kwargs: Any
) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    api_version: str = kwargs.pop("api_version", _params.pop("api-version", "2023-12-01"))
    content_type: Optional[str] = kwargs.pop("content_type", _headers.pop("Content-Type", None))
    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = kwargs.pop(
        "template_url",
        "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.AppPlatform/Spring/{serviceName}/buildServices/{buildServiceName}/builds/{buildName}",
    )  # pylint: disable=line-too-long
    path_format_arguments = {
        "subscriptionId": _SERIALIZER.url("subscription_id", subscription_id, "str"),
        "resourceGroupName": _SERIALIZER.url("resource_group_name", resource_group_name, "str"),
        "serviceName": _SERIALIZER.url("service_name", service_name, "str", pattern=r"^[a-z][a-z0-9-]*[a-z0-9]$"),
        "buildServiceName": _SERIALIZER.url("build_service_name", build_service_name, "str"),
        "buildName": _SERIALIZER.url("build_name", build_name, "str"),
    }

    _url: str = _url.format(**path_format_arguments)  # type: ignore

    # Construct parameters
    _params["api-version"] = _SERIALIZER.query("api_version", api_version, "str")

    # Construct headers
    if content_type is not None:
        _headers["Content-Type"] = _SERIALIZER.header("content_type", content_type, "str")
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="PUT", url=_url, params=_params, headers=_headers, **kwargs)


def build_delete_build_request(
    resource_group_name: str,
    service_name: str,
    build_service_name: str,
    build_name: str,
    subscription_id: str,
    **kwargs: Any
) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    api_version: str = kwargs.pop("api_version", _params.pop("api-version", "2023-12-01"))
    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = kwargs.pop(
        "template_url",
        "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.AppPlatform/Spring/{serviceName}/buildServices/{buildServiceName}/builds/{buildName}",
    )  # pylint: disable=line-too-long
    path_format_arguments = {
        "subscriptionId": _SERIALIZER.url("subscription_id", subscription_id, "str"),
        "resourceGroupName": _SERIALIZER.url("resource_group_name", resource_group_name, "str"),
        "serviceName": _SERIALIZER.url("service_name", service_name, "str", pattern=r"^[a-z][a-z0-9-]*[a-z0-9]$"),
        "buildServiceName": _SERIALIZER.url("build_service_name", build_service_name, "str"),
        "buildName": _SERIALIZER.url("build_name", build_name, "str"),
    }

    _url: str = _url.format(**path_format_arguments)  # type: ignore

    # Construct parameters
    _params["api-version"] = _SERIALIZER.query("api_version", api_version, "str")

    # Construct headers
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="DELETE", url=_url, params=_params, headers=_headers, **kwargs)


def build_list_build_results_request(
    resource_group_name: str,
    service_name: str,
    build_service_name: str,
    build_name: str,
    subscription_id: str,
    **kwargs: Any
) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    api_version: str = kwargs.pop("api_version", _params.pop("api-version", "2023-12-01"))
    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = kwargs.pop(
        "template_url",
        "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.AppPlatform/Spring/{serviceName}/buildServices/{buildServiceName}/builds/{buildName}/results",
    )  # pylint: disable=line-too-long
    path_format_arguments = {
        "subscriptionId": _SERIALIZER.url("subscription_id", subscription_id, "str"),
        "resourceGroupName": _SERIALIZER.url("resource_group_name", resource_group_name, "str"),
        "serviceName": _SERIALIZER.url("service_name", service_name, "str", pattern=r"^[a-z][a-z0-9-]*[a-z0-9]$"),
        "buildServiceName": _SERIALIZER.url("build_service_name", build_service_name, "str"),
        "buildName": _SERIALIZER.url("build_name", build_name, "str"),
    }

    _url: str = _url.format(**path_format_arguments)  # type: ignore

    # Construct parameters
    _params["api-version"] = _SERIALIZER.query("api_version", api_version, "str")

    # Construct headers
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="GET", url=_url, params=_params, headers=_headers, **kwargs)


def build_get_build_result_request(
    resource_group_name: str,
    service_name: str,
    build_service_name: str,
    build_name: str,
    build_result_name: str,
    subscription_id: str,
    **kwargs: Any
) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    api_version: str = kwargs.pop("api_version", _params.pop("api-version", "2023-12-01"))
    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = kwargs.pop(
        "template_url",
        "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.AppPlatform/Spring/{serviceName}/buildServices/{buildServiceName}/builds/{buildName}/results/{buildResultName}",
    )  # pylint: disable=line-too-long
    path_format_arguments = {
        "subscriptionId": _SERIALIZER.url("subscription_id", subscription_id, "str"),
        "resourceGroupName": _SERIALIZER.url("resource_group_name", resource_group_name, "str"),
        "serviceName": _SERIALIZER.url("service_name", service_name, "str", pattern=r"^[a-z][a-z0-9-]*[a-z0-9]$"),
        "buildServiceName": _SERIALIZER.url("build_service_name", build_service_name, "str"),
        "buildName": _SERIALIZER.url("build_name", build_name, "str"),
        "buildResultName": _SERIALIZER.url("build_result_name", build_result_name, "str"),
    }

    _url: str = _url.format(**path_format_arguments)  # type: ignore

    # Construct parameters
    _params["api-version"] = _SERIALIZER.query("api_version", api_version, "str")

    # Construct headers
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="GET", url=_url, params=_params, headers=_headers, **kwargs)


def build_get_build_result_log_request(
    resource_group_name: str,
    service_name: str,
    build_service_name: str,
    build_name: str,
    build_result_name: str,
    subscription_id: str,
    **kwargs: Any
) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    api_version: str = kwargs.pop("api_version", _params.pop("api-version", "2023-12-01"))
    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = kwargs.pop(
        "template_url",
        "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.AppPlatform/Spring/{serviceName}/buildServices/{buildServiceName}/builds/{buildName}/results/{buildResultName}/getLogFileUrl",
    )  # pylint: disable=line-too-long
    path_format_arguments = {
        "subscriptionId": _SERIALIZER.url("subscription_id", subscription_id, "str"),
        "resourceGroupName": _SERIALIZER.url("resource_group_name", resource_group_name, "str"),
        "serviceName": _SERIALIZER.url("service_name", service_name, "str", pattern=r"^[a-z][a-z0-9-]*[a-z0-9]$"),
        "buildServiceName": _SERIALIZER.url("build_service_name", build_service_name, "str"),
        "buildName": _SERIALIZER.url("build_name", build_name, "str"),
        "buildResultName": _SERIALIZER.url("build_result_name", build_result_name, "str"),
    }

    _url: str = _url.format(**path_format_arguments)  # type: ignore

    # Construct parameters
    _params["api-version"] = _SERIALIZER.query("api_version", api_version, "str")

    # Construct headers
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="POST", url=_url, params=_params, headers=_headers, **kwargs)


def build_get_resource_upload_url_request(
    resource_group_name: str, service_name: str, build_service_name: str, subscription_id: str, **kwargs: Any
) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    api_version: str = kwargs.pop("api_version", _params.pop("api-version", "2023-12-01"))
    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = kwargs.pop(
        "template_url",
        "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.AppPlatform/Spring/{serviceName}/buildServices/{buildServiceName}/getResourceUploadUrl",
    )  # pylint: disable=line-too-long
    path_format_arguments = {
        "subscriptionId": _SERIALIZER.url("subscription_id", subscription_id, "str"),
        "resourceGroupName": _SERIALIZER.url("resource_group_name", resource_group_name, "str"),
        "serviceName": _SERIALIZER.url("service_name", service_name, "str", pattern=r"^[a-z][a-z0-9-]*[a-z0-9]$"),
        "buildServiceName": _SERIALIZER.url("build_service_name", build_service_name, "str"),
    }

    _url: str = _url.format(**path_format_arguments)  # type: ignore

    # Construct parameters
    _params["api-version"] = _SERIALIZER.query("api_version", api_version, "str")

    # Construct headers
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="POST", url=_url, params=_params, headers=_headers, **kwargs)


def build_list_supported_buildpacks_request(
    resource_group_name: str, service_name: str, build_service_name: str, subscription_id: str, **kwargs: Any
) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    api_version: str = kwargs.pop("api_version", _params.pop("api-version", "2023-12-01"))
    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = kwargs.pop(
        "template_url",
        "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.AppPlatform/Spring/{serviceName}/buildServices/{buildServiceName}/supportedBuildpacks",
    )  # pylint: disable=line-too-long
    path_format_arguments = {
        "subscriptionId": _SERIALIZER.url("subscription_id", subscription_id, "str"),
        "resourceGroupName": _SERIALIZER.url("resource_group_name", resource_group_name, "str"),
        "serviceName": _SERIALIZER.url("service_name", service_name, "str", pattern=r"^[a-z][a-z0-9-]*[a-z0-9]$"),
        "buildServiceName": _SERIALIZER.url("build_service_name", build_service_name, "str"),
    }

    _url: str = _url.format(**path_format_arguments)  # type: ignore

    # Construct parameters
    _params["api-version"] = _SERIALIZER.query("api_version", api_version, "str")

    # Construct headers
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="GET", url=_url, params=_params, headers=_headers, **kwargs)


def build_get_supported_buildpack_request(
    resource_group_name: str,
    service_name: str,
    build_service_name: str,
    buildpack_name: str,
    subscription_id: str,
    **kwargs: Any
) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    api_version: str = kwargs.pop("api_version", _params.pop("api-version", "2023-12-01"))
    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = kwargs.pop(
        "template_url",
        "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.AppPlatform/Spring/{serviceName}/buildServices/{buildServiceName}/supportedBuildpacks/{buildpackName}",
    )  # pylint: disable=line-too-long
    path_format_arguments = {
        "subscriptionId": _SERIALIZER.url("subscription_id", subscription_id, "str"),
        "resourceGroupName": _SERIALIZER.url("resource_group_name", resource_group_name, "str"),
        "serviceName": _SERIALIZER.url("service_name", service_name, "str", pattern=r"^[a-z][a-z0-9-]*[a-z0-9]$"),
        "buildServiceName": _SERIALIZER.url("build_service_name", build_service_name, "str"),
        "buildpackName": _SERIALIZER.url("buildpack_name", buildpack_name, "str"),
    }

    _url: str = _url.format(**path_format_arguments)  # type: ignore

    # Construct parameters
    _params["api-version"] = _SERIALIZER.query("api_version", api_version, "str")

    # Construct headers
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="GET", url=_url, params=_params, headers=_headers, **kwargs)


def build_list_supported_stacks_request(
    resource_group_name: str, service_name: str, build_service_name: str, subscription_id: str, **kwargs: Any
) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    api_version: str = kwargs.pop("api_version", _params.pop("api-version", "2023-12-01"))
    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = kwargs.pop(
        "template_url",
        "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.AppPlatform/Spring/{serviceName}/buildServices/{buildServiceName}/supportedStacks",
    )  # pylint: disable=line-too-long
    path_format_arguments = {
        "subscriptionId": _SERIALIZER.url("subscription_id", subscription_id, "str"),
        "resourceGroupName": _SERIALIZER.url("resource_group_name", resource_group_name, "str"),
        "serviceName": _SERIALIZER.url("service_name", service_name, "str", pattern=r"^[a-z][a-z0-9-]*[a-z0-9]$"),
        "buildServiceName": _SERIALIZER.url("build_service_name", build_service_name, "str"),
    }

    _url: str = _url.format(**path_format_arguments)  # type: ignore

    # Construct parameters
    _params["api-version"] = _SERIALIZER.query("api_version", api_version, "str")

    # Construct headers
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="GET", url=_url, params=_params, headers=_headers, **kwargs)


def build_get_supported_stack_request(
    resource_group_name: str,
    service_name: str,
    build_service_name: str,
    stack_name: str,
    subscription_id: str,
    **kwargs: Any
) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    api_version: str = kwargs.pop("api_version", _params.pop("api-version", "2023-12-01"))
    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = kwargs.pop(
        "template_url",
        "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.AppPlatform/Spring/{serviceName}/buildServices/{buildServiceName}/supportedStacks/{stackName}",
    )  # pylint: disable=line-too-long
    path_format_arguments = {
        "subscriptionId": _SERIALIZER.url("subscription_id", subscription_id, "str"),
        "resourceGroupName": _SERIALIZER.url("resource_group_name", resource_group_name, "str"),
        "serviceName": _SERIALIZER.url("service_name", service_name, "str", pattern=r"^[a-z][a-z0-9-]*[a-z0-9]$"),
        "buildServiceName": _SERIALIZER.url("build_service_name", build_service_name, "str"),
        "stackName": _SERIALIZER.url("stack_name", stack_name, "str"),
    }

    _url: str = _url.format(**path_format_arguments)  # type: ignore

    # Construct parameters
    _params["api-version"] = _SERIALIZER.query("api_version", api_version, "str")

    # Construct headers
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="GET", url=_url, params=_params, headers=_headers, **kwargs)


class BuildServiceOperations:
    """
    .. warning::
        **DO NOT** instantiate this class directly.

        Instead, you should access the following operations through
        :class:`~azure.mgmt.appplatform.AppPlatformManagementClient`'s
        :attr:`build_service` attribute.
    """

    models = _models

    def __init__(self, *args, **kwargs):
        input_args = list(args)
        self._client = input_args.pop(0) if input_args else kwargs.pop("client")
        self._config = input_args.pop(0) if input_args else kwargs.pop("config")
        self._serialize = input_args.pop(0) if input_args else kwargs.pop("serializer")
        self._deserialize = input_args.pop(0) if input_args else kwargs.pop("deserializer")

    @distributed_trace
    def list_build_services(
        self, resource_group_name: str, service_name: str, **kwargs: Any
    ) -> Iterable["_models.BuildService"]:
        """List build services resource.

        :param resource_group_name: The name of the resource group that contains the resource. You can
         obtain this value from the Azure Resource Manager API or the portal. Required.
        :type resource_group_name: str
        :param service_name: The name of the Service resource. Required.
        :type service_name: str
        :return: An iterator like instance of either BuildService or the result of cls(response)
        :rtype: ~azure.core.paging.ItemPaged[~azure.mgmt.appplatform.models.BuildService]
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        _headers = kwargs.pop("headers", {}) or {}
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop("api_version", _params.pop("api-version", self._config.api_version))
        cls: ClsType[_models.BuildServiceCollection] = kwargs.pop("cls", None)

        error_map: MutableMapping[int, Type[HttpResponseError]] = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        def prepare_request(next_link=None):
            if not next_link:

                _request = build_list_build_services_request(
                    resource_group_name=resource_group_name,
                    service_name=service_name,
                    subscription_id=self._config.subscription_id,
                    api_version=api_version,
                    headers=_headers,
                    params=_params,
                )
                _request.url = self._client.format_url(_request.url)

            else:
                # make call to next link with the client's api-version
                _parsed_next_link = urllib.parse.urlparse(next_link)
                _next_request_params = case_insensitive_dict(
                    {
                        key: [urllib.parse.quote(v) for v in value]
                        for key, value in urllib.parse.parse_qs(_parsed_next_link.query).items()
                    }
                )
                _next_request_params["api-version"] = self._config.api_version
                _request = HttpRequest(
                    "GET", urllib.parse.urljoin(next_link, _parsed_next_link.path), params=_next_request_params
                )
                _request.url = self._client.format_url(_request.url)
                _request.method = "GET"
            return _request

        def extract_data(pipeline_response):
            deserialized = self._deserialize("BuildServiceCollection", pipeline_response)
            list_of_elem = deserialized.value
            if cls:
                list_of_elem = cls(list_of_elem)  # type: ignore
            return deserialized.next_link or None, iter(list_of_elem)

        def get_next(next_link=None):
            _request = prepare_request(next_link)

            _stream = False
            pipeline_response: PipelineResponse = self._client._pipeline.run(  # pylint: disable=protected-access
                _request, stream=_stream, **kwargs
            )
            response = pipeline_response.http_response

            if response.status_code not in [200]:
                map_error(status_code=response.status_code, response=response, error_map=error_map)
                raise HttpResponseError(response=response, error_format=ARMErrorFormat)

            return pipeline_response

        return ItemPaged(get_next, extract_data)

    @distributed_trace
    def get_build_service(
        self, resource_group_name: str, service_name: str, build_service_name: str, **kwargs: Any
    ) -> _models.BuildService:
        """Get a build service resource.

        :param resource_group_name: The name of the resource group that contains the resource. You can
         obtain this value from the Azure Resource Manager API or the portal. Required.
        :type resource_group_name: str
        :param service_name: The name of the Service resource. Required.
        :type service_name: str
        :param build_service_name: The name of the build service resource. Required.
        :type build_service_name: str
        :return: BuildService or the result of cls(response)
        :rtype: ~azure.mgmt.appplatform.models.BuildService
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map: MutableMapping[int, Type[HttpResponseError]] = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop("api_version", _params.pop("api-version", self._config.api_version))
        cls: ClsType[_models.BuildService] = kwargs.pop("cls", None)

        _request = build_get_build_service_request(
            resource_group_name=resource_group_name,
            service_name=service_name,
            build_service_name=build_service_name,
            subscription_id=self._config.subscription_id,
            api_version=api_version,
            headers=_headers,
            params=_params,
        )
        _request.url = self._client.format_url(_request.url)

        _stream = False
        pipeline_response: PipelineResponse = self._client._pipeline.run(  # pylint: disable=protected-access
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response, error_format=ARMErrorFormat)

        deserialized = self._deserialize("BuildService", pipeline_response.http_response)

        if cls:
            return cls(pipeline_response, deserialized, {})  # type: ignore

        return deserialized  # type: ignore

    def _create_or_update_initial(
        self,
        resource_group_name: str,
        service_name: str,
        build_service_name: str,
        build_service: Union[_models.BuildService, IO[bytes]],
        **kwargs: Any
    ) -> Iterator[bytes]:
        error_map: MutableMapping[int, Type[HttpResponseError]] = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop("api_version", _params.pop("api-version", self._config.api_version))
        content_type: Optional[str] = kwargs.pop("content_type", _headers.pop("Content-Type", None))
        cls: ClsType[Iterator[bytes]] = kwargs.pop("cls", None)

        content_type = content_type or "application/json"
        _json = None
        _content = None
        if isinstance(build_service, (IOBase, bytes)):
            _content = build_service
        else:
            _json = self._serialize.body(build_service, "BuildService")

        _request = build_create_or_update_request(
            resource_group_name=resource_group_name,
            service_name=service_name,
            build_service_name=build_service_name,
            subscription_id=self._config.subscription_id,
            api_version=api_version,
            content_type=content_type,
            json=_json,
            content=_content,
            headers=_headers,
            params=_params,
        )
        _request.url = self._client.format_url(_request.url)

        _decompress = kwargs.pop("decompress", True)
        _stream = True
        pipeline_response: PipelineResponse = self._client._pipeline.run(  # pylint: disable=protected-access
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200, 201]:
            try:
                response.read()  # Load the body in memory and close the socket
            except (StreamConsumedError, StreamClosedError):
                pass
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response, error_format=ARMErrorFormat)

        deserialized = response.stream_download(self._client._pipeline, decompress=_decompress)

        if cls:
            return cls(pipeline_response, deserialized, {})  # type: ignore

        return deserialized  # type: ignore

    @overload
    def begin_create_or_update(
        self,
        resource_group_name: str,
        service_name: str,
        build_service_name: str,
        build_service: _models.BuildService,
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> LROPoller[_models.BuildService]:
        """Create a build service resource.

        :param resource_group_name: The name of the resource group that contains the resource. You can
         obtain this value from the Azure Resource Manager API or the portal. Required.
        :type resource_group_name: str
        :param service_name: The name of the Service resource. Required.
        :type service_name: str
        :param build_service_name: The name of the build service resource. Required.
        :type build_service_name: str
        :param build_service: Parameters for the create operation. Required.
        :type build_service: ~azure.mgmt.appplatform.models.BuildService
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: An instance of LROPoller that returns either BuildService or the result of
         cls(response)
        :rtype: ~azure.core.polling.LROPoller[~azure.mgmt.appplatform.models.BuildService]
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    def begin_create_or_update(
        self,
        resource_group_name: str,
        service_name: str,
        build_service_name: str,
        build_service: IO[bytes],
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> LROPoller[_models.BuildService]:
        """Create a build service resource.

        :param resource_group_name: The name of the resource group that contains the resource. You can
         obtain this value from the Azure Resource Manager API or the portal. Required.
        :type resource_group_name: str
        :param service_name: The name of the Service resource. Required.
        :type service_name: str
        :param build_service_name: The name of the build service resource. Required.
        :type build_service_name: str
        :param build_service: Parameters for the create operation. Required.
        :type build_service: IO[bytes]
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: An instance of LROPoller that returns either BuildService or the result of
         cls(response)
        :rtype: ~azure.core.polling.LROPoller[~azure.mgmt.appplatform.models.BuildService]
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @distributed_trace
    def begin_create_or_update(
        self,
        resource_group_name: str,
        service_name: str,
        build_service_name: str,
        build_service: Union[_models.BuildService, IO[bytes]],
        **kwargs: Any
    ) -> LROPoller[_models.BuildService]:
        """Create a build service resource.

        :param resource_group_name: The name of the resource group that contains the resource. You can
         obtain this value from the Azure Resource Manager API or the portal. Required.
        :type resource_group_name: str
        :param service_name: The name of the Service resource. Required.
        :type service_name: str
        :param build_service_name: The name of the build service resource. Required.
        :type build_service_name: str
        :param build_service: Parameters for the create operation. Is either a BuildService type or a
         IO[bytes] type. Required.
        :type build_service: ~azure.mgmt.appplatform.models.BuildService or IO[bytes]
        :return: An instance of LROPoller that returns either BuildService or the result of
         cls(response)
        :rtype: ~azure.core.polling.LROPoller[~azure.mgmt.appplatform.models.BuildService]
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop("api_version", _params.pop("api-version", self._config.api_version))
        content_type: Optional[str] = kwargs.pop("content_type", _headers.pop("Content-Type", None))
        cls: ClsType[_models.BuildService] = kwargs.pop("cls", None)
        polling: Union[bool, PollingMethod] = kwargs.pop("polling", True)
        lro_delay = kwargs.pop("polling_interval", self._config.polling_interval)
        cont_token: Optional[str] = kwargs.pop("continuation_token", None)
        if cont_token is None:
            raw_result = self._create_or_update_initial(
                resource_group_name=resource_group_name,
                service_name=service_name,
                build_service_name=build_service_name,
                build_service=build_service,
                api_version=api_version,
                content_type=content_type,
                cls=lambda x, y, z: x,
                headers=_headers,
                params=_params,
                **kwargs
            )
            raw_result.http_response.read()  # type: ignore
        kwargs.pop("error_map", None)

        def get_long_running_output(pipeline_response):
            deserialized = self._deserialize("BuildService", pipeline_response.http_response)
            if cls:
                return cls(pipeline_response, deserialized, {})  # type: ignore
            return deserialized

        if polling is True:
            polling_method: PollingMethod = cast(
                PollingMethod, ARMPolling(lro_delay, lro_options={"final-state-via": "location"}, **kwargs)
            )
        elif polling is False:
            polling_method = cast(PollingMethod, NoPolling())
        else:
            polling_method = polling
        if cont_token:
            return LROPoller[_models.BuildService].from_continuation_token(
                polling_method=polling_method,
                continuation_token=cont_token,
                client=self._client,
                deserialization_callback=get_long_running_output,
            )
        return LROPoller[_models.BuildService](
            self._client, raw_result, get_long_running_output, polling_method  # type: ignore
        )

    @distributed_trace
    def list_builds(
        self, resource_group_name: str, service_name: str, build_service_name: str, **kwargs: Any
    ) -> Iterable["_models.Build"]:
        """List KPack builds.

        :param resource_group_name: The name of the resource group that contains the resource. You can
         obtain this value from the Azure Resource Manager API or the portal. Required.
        :type resource_group_name: str
        :param service_name: The name of the Service resource. Required.
        :type service_name: str
        :param build_service_name: The name of the build service resource. Required.
        :type build_service_name: str
        :return: An iterator like instance of either Build or the result of cls(response)
        :rtype: ~azure.core.paging.ItemPaged[~azure.mgmt.appplatform.models.Build]
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        _headers = kwargs.pop("headers", {}) or {}
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop("api_version", _params.pop("api-version", self._config.api_version))
        cls: ClsType[_models.BuildCollection] = kwargs.pop("cls", None)

        error_map: MutableMapping[int, Type[HttpResponseError]] = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        def prepare_request(next_link=None):
            if not next_link:

                _request = build_list_builds_request(
                    resource_group_name=resource_group_name,
                    service_name=service_name,
                    build_service_name=build_service_name,
                    subscription_id=self._config.subscription_id,
                    api_version=api_version,
                    headers=_headers,
                    params=_params,
                )
                _request.url = self._client.format_url(_request.url)

            else:
                # make call to next link with the client's api-version
                _parsed_next_link = urllib.parse.urlparse(next_link)
                _next_request_params = case_insensitive_dict(
                    {
                        key: [urllib.parse.quote(v) for v in value]
                        for key, value in urllib.parse.parse_qs(_parsed_next_link.query).items()
                    }
                )
                _next_request_params["api-version"] = self._config.api_version
                _request = HttpRequest(
                    "GET", urllib.parse.urljoin(next_link, _parsed_next_link.path), params=_next_request_params
                )
                _request.url = self._client.format_url(_request.url)
                _request.method = "GET"
            return _request

        def extract_data(pipeline_response):
            deserialized = self._deserialize("BuildCollection", pipeline_response)
            list_of_elem = deserialized.value
            if cls:
                list_of_elem = cls(list_of_elem)  # type: ignore
            return deserialized.next_link or None, iter(list_of_elem)

        def get_next(next_link=None):
            _request = prepare_request(next_link)

            _stream = False
            pipeline_response: PipelineResponse = self._client._pipeline.run(  # pylint: disable=protected-access
                _request, stream=_stream, **kwargs
            )
            response = pipeline_response.http_response

            if response.status_code not in [200]:
                map_error(status_code=response.status_code, response=response, error_map=error_map)
                raise HttpResponseError(response=response, error_format=ARMErrorFormat)

            return pipeline_response

        return ItemPaged(get_next, extract_data)

    @distributed_trace
    def get_build(
        self, resource_group_name: str, service_name: str, build_service_name: str, build_name: str, **kwargs: Any
    ) -> _models.Build:
        """Get a KPack build.

        :param resource_group_name: The name of the resource group that contains the resource. You can
         obtain this value from the Azure Resource Manager API or the portal. Required.
        :type resource_group_name: str
        :param service_name: The name of the Service resource. Required.
        :type service_name: str
        :param build_service_name: The name of the build service resource. Required.
        :type build_service_name: str
        :param build_name: The name of the build resource. Required.
        :type build_name: str
        :return: Build or the result of cls(response)
        :rtype: ~azure.mgmt.appplatform.models.Build
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map: MutableMapping[int, Type[HttpResponseError]] = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop("api_version", _params.pop("api-version", self._config.api_version))
        cls: ClsType[_models.Build] = kwargs.pop("cls", None)

        _request = build_get_build_request(
            resource_group_name=resource_group_name,
            service_name=service_name,
            build_service_name=build_service_name,
            build_name=build_name,
            subscription_id=self._config.subscription_id,
            api_version=api_version,
            headers=_headers,
            params=_params,
        )
        _request.url = self._client.format_url(_request.url)

        _stream = False
        pipeline_response: PipelineResponse = self._client._pipeline.run(  # pylint: disable=protected-access
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response, error_format=ARMErrorFormat)

        deserialized = self._deserialize("Build", pipeline_response.http_response)

        if cls:
            return cls(pipeline_response, deserialized, {})  # type: ignore

        return deserialized  # type: ignore

    @overload
    def create_or_update_build(
        self,
        resource_group_name: str,
        service_name: str,
        build_service_name: str,
        build_name: str,
        build: _models.Build,
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> _models.Build:
        """Create or update a KPack build.

        :param resource_group_name: The name of the resource group that contains the resource. You can
         obtain this value from the Azure Resource Manager API or the portal. Required.
        :type resource_group_name: str
        :param service_name: The name of the Service resource. Required.
        :type service_name: str
        :param build_service_name: The name of the build service resource. Required.
        :type build_service_name: str
        :param build_name: The name of the build resource. Required.
        :type build_name: str
        :param build: Parameters for the create or update operation. Required.
        :type build: ~azure.mgmt.appplatform.models.Build
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: Build or the result of cls(response)
        :rtype: ~azure.mgmt.appplatform.models.Build
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    def create_or_update_build(
        self,
        resource_group_name: str,
        service_name: str,
        build_service_name: str,
        build_name: str,
        build: IO[bytes],
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> _models.Build:
        """Create or update a KPack build.

        :param resource_group_name: The name of the resource group that contains the resource. You can
         obtain this value from the Azure Resource Manager API or the portal. Required.
        :type resource_group_name: str
        :param service_name: The name of the Service resource. Required.
        :type service_name: str
        :param build_service_name: The name of the build service resource. Required.
        :type build_service_name: str
        :param build_name: The name of the build resource. Required.
        :type build_name: str
        :param build: Parameters for the create or update operation. Required.
        :type build: IO[bytes]
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: Build or the result of cls(response)
        :rtype: ~azure.mgmt.appplatform.models.Build
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @distributed_trace
    def create_or_update_build(
        self,
        resource_group_name: str,
        service_name: str,
        build_service_name: str,
        build_name: str,
        build: Union[_models.Build, IO[bytes]],
        **kwargs: Any
    ) -> _models.Build:
        """Create or update a KPack build.

        :param resource_group_name: The name of the resource group that contains the resource. You can
         obtain this value from the Azure Resource Manager API or the portal. Required.
        :type resource_group_name: str
        :param service_name: The name of the Service resource. Required.
        :type service_name: str
        :param build_service_name: The name of the build service resource. Required.
        :type build_service_name: str
        :param build_name: The name of the build resource. Required.
        :type build_name: str
        :param build: Parameters for the create or update operation. Is either a Build type or a
         IO[bytes] type. Required.
        :type build: ~azure.mgmt.appplatform.models.Build or IO[bytes]
        :return: Build or the result of cls(response)
        :rtype: ~azure.mgmt.appplatform.models.Build
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map: MutableMapping[int, Type[HttpResponseError]] = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop("api_version", _params.pop("api-version", self._config.api_version))
        content_type: Optional[str] = kwargs.pop("content_type", _headers.pop("Content-Type", None))
        cls: ClsType[_models.Build] = kwargs.pop("cls", None)

        content_type = content_type or "application/json"
        _json = None
        _content = None
        if isinstance(build, (IOBase, bytes)):
            _content = build
        else:
            _json = self._serialize.body(build, "Build")

        _request = build_create_or_update_build_request(
            resource_group_name=resource_group_name,
            service_name=service_name,
            build_service_name=build_service_name,
            build_name=build_name,
            subscription_id=self._config.subscription_id,
            api_version=api_version,
            content_type=content_type,
            json=_json,
            content=_content,
            headers=_headers,
            params=_params,
        )
        _request.url = self._client.format_url(_request.url)

        _stream = False
        pipeline_response: PipelineResponse = self._client._pipeline.run(  # pylint: disable=protected-access
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200, 201]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response, error_format=ARMErrorFormat)

        deserialized = self._deserialize("Build", pipeline_response.http_response)

        if cls:
            return cls(pipeline_response, deserialized, {})  # type: ignore

        return deserialized  # type: ignore

    def _delete_build_initial(
        self, resource_group_name: str, service_name: str, build_service_name: str, build_name: str, **kwargs: Any
    ) -> Iterator[bytes]:
        error_map: MutableMapping[int, Type[HttpResponseError]] = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop("api_version", _params.pop("api-version", self._config.api_version))
        cls: ClsType[Iterator[bytes]] = kwargs.pop("cls", None)

        _request = build_delete_build_request(
            resource_group_name=resource_group_name,
            service_name=service_name,
            build_service_name=build_service_name,
            build_name=build_name,
            subscription_id=self._config.subscription_id,
            api_version=api_version,
            headers=_headers,
            params=_params,
        )
        _request.url = self._client.format_url(_request.url)

        _decompress = kwargs.pop("decompress", True)
        _stream = True
        pipeline_response: PipelineResponse = self._client._pipeline.run(  # pylint: disable=protected-access
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [202, 204]:
            try:
                response.read()  # Load the body in memory and close the socket
            except (StreamConsumedError, StreamClosedError):
                pass
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response, error_format=ARMErrorFormat)

        response_headers = {}
        if response.status_code == 202:
            response_headers["Location"] = self._deserialize("str", response.headers.get("Location"))

        deserialized = response.stream_download(self._client._pipeline, decompress=_decompress)

        if cls:
            return cls(pipeline_response, deserialized, response_headers)  # type: ignore

        return deserialized  # type: ignore

    @distributed_trace
    def begin_delete_build(
        self, resource_group_name: str, service_name: str, build_service_name: str, build_name: str, **kwargs: Any
    ) -> LROPoller[None]:
        """delete a KPack build.

        :param resource_group_name: The name of the resource group that contains the resource. You can
         obtain this value from the Azure Resource Manager API or the portal. Required.
        :type resource_group_name: str
        :param service_name: The name of the Service resource. Required.
        :type service_name: str
        :param build_service_name: The name of the build service resource. Required.
        :type build_service_name: str
        :param build_name: The name of the build resource. Required.
        :type build_name: str
        :return: An instance of LROPoller that returns either None or the result of cls(response)
        :rtype: ~azure.core.polling.LROPoller[None]
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        _headers = kwargs.pop("headers", {}) or {}
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop("api_version", _params.pop("api-version", self._config.api_version))
        cls: ClsType[None] = kwargs.pop("cls", None)
        polling: Union[bool, PollingMethod] = kwargs.pop("polling", True)
        lro_delay = kwargs.pop("polling_interval", self._config.polling_interval)
        cont_token: Optional[str] = kwargs.pop("continuation_token", None)
        if cont_token is None:
            raw_result = self._delete_build_initial(
                resource_group_name=resource_group_name,
                service_name=service_name,
                build_service_name=build_service_name,
                build_name=build_name,
                api_version=api_version,
                cls=lambda x, y, z: x,
                headers=_headers,
                params=_params,
                **kwargs
            )
            raw_result.http_response.read()  # type: ignore
        kwargs.pop("error_map", None)

        def get_long_running_output(pipeline_response):  # pylint: disable=inconsistent-return-statements
            if cls:
                return cls(pipeline_response, None, {})  # type: ignore

        if polling is True:
            polling_method: PollingMethod = cast(
                PollingMethod, ARMPolling(lro_delay, lro_options={"final-state-via": "location"}, **kwargs)
            )
        elif polling is False:
            polling_method = cast(PollingMethod, NoPolling())
        else:
            polling_method = polling
        if cont_token:
            return LROPoller[None].from_continuation_token(
                polling_method=polling_method,
                continuation_token=cont_token,
                client=self._client,
                deserialization_callback=get_long_running_output,
            )
        return LROPoller[None](self._client, raw_result, get_long_running_output, polling_method)  # type: ignore

    @distributed_trace
    def list_build_results(
        self, resource_group_name: str, service_name: str, build_service_name: str, build_name: str, **kwargs: Any
    ) -> Iterable["_models.BuildResult"]:
        """List KPack build results.

        :param resource_group_name: The name of the resource group that contains the resource. You can
         obtain this value from the Azure Resource Manager API or the portal. Required.
        :type resource_group_name: str
        :param service_name: The name of the Service resource. Required.
        :type service_name: str
        :param build_service_name: The name of the build service resource. Required.
        :type build_service_name: str
        :param build_name: The name of the build resource. Required.
        :type build_name: str
        :return: An iterator like instance of either BuildResult or the result of cls(response)
        :rtype: ~azure.core.paging.ItemPaged[~azure.mgmt.appplatform.models.BuildResult]
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        _headers = kwargs.pop("headers", {}) or {}
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop("api_version", _params.pop("api-version", self._config.api_version))
        cls: ClsType[_models.BuildResultCollection] = kwargs.pop("cls", None)

        error_map: MutableMapping[int, Type[HttpResponseError]] = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        def prepare_request(next_link=None):
            if not next_link:

                _request = build_list_build_results_request(
                    resource_group_name=resource_group_name,
                    service_name=service_name,
                    build_service_name=build_service_name,
                    build_name=build_name,
                    subscription_id=self._config.subscription_id,
                    api_version=api_version,
                    headers=_headers,
                    params=_params,
                )
                _request.url = self._client.format_url(_request.url)

            else:
                # make call to next link with the client's api-version
                _parsed_next_link = urllib.parse.urlparse(next_link)
                _next_request_params = case_insensitive_dict(
                    {
                        key: [urllib.parse.quote(v) for v in value]
                        for key, value in urllib.parse.parse_qs(_parsed_next_link.query).items()
                    }
                )
                _next_request_params["api-version"] = self._config.api_version
                _request = HttpRequest(
                    "GET", urllib.parse.urljoin(next_link, _parsed_next_link.path), params=_next_request_params
                )
                _request.url = self._client.format_url(_request.url)
                _request.method = "GET"
            return _request

        def extract_data(pipeline_response):
            deserialized = self._deserialize("BuildResultCollection", pipeline_response)
            list_of_elem = deserialized.value
            if cls:
                list_of_elem = cls(list_of_elem)  # type: ignore
            return deserialized.next_link or None, iter(list_of_elem)

        def get_next(next_link=None):
            _request = prepare_request(next_link)

            _stream = False
            pipeline_response: PipelineResponse = self._client._pipeline.run(  # pylint: disable=protected-access
                _request, stream=_stream, **kwargs
            )
            response = pipeline_response.http_response

            if response.status_code not in [200]:
                map_error(status_code=response.status_code, response=response, error_map=error_map)
                raise HttpResponseError(response=response, error_format=ARMErrorFormat)

            return pipeline_response

        return ItemPaged(get_next, extract_data)

    @distributed_trace
    def get_build_result(
        self,
        resource_group_name: str,
        service_name: str,
        build_service_name: str,
        build_name: str,
        build_result_name: str,
        **kwargs: Any
    ) -> _models.BuildResult:
        """Get a KPack build result.

        :param resource_group_name: The name of the resource group that contains the resource. You can
         obtain this value from the Azure Resource Manager API or the portal. Required.
        :type resource_group_name: str
        :param service_name: The name of the Service resource. Required.
        :type service_name: str
        :param build_service_name: The name of the build service resource. Required.
        :type build_service_name: str
        :param build_name: The name of the build resource. Required.
        :type build_name: str
        :param build_result_name: The name of the build result resource. Required.
        :type build_result_name: str
        :return: BuildResult or the result of cls(response)
        :rtype: ~azure.mgmt.appplatform.models.BuildResult
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map: MutableMapping[int, Type[HttpResponseError]] = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop("api_version", _params.pop("api-version", self._config.api_version))
        cls: ClsType[_models.BuildResult] = kwargs.pop("cls", None)

        _request = build_get_build_result_request(
            resource_group_name=resource_group_name,
            service_name=service_name,
            build_service_name=build_service_name,
            build_name=build_name,
            build_result_name=build_result_name,
            subscription_id=self._config.subscription_id,
            api_version=api_version,
            headers=_headers,
            params=_params,
        )
        _request.url = self._client.format_url(_request.url)

        _stream = False
        pipeline_response: PipelineResponse = self._client._pipeline.run(  # pylint: disable=protected-access
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response, error_format=ARMErrorFormat)

        deserialized = self._deserialize("BuildResult", pipeline_response.http_response)

        if cls:
            return cls(pipeline_response, deserialized, {})  # type: ignore

        return deserialized  # type: ignore

    @distributed_trace
    def get_build_result_log(
        self,
        resource_group_name: str,
        service_name: str,
        build_service_name: str,
        build_name: str,
        build_result_name: str,
        **kwargs: Any
    ) -> _models.BuildResultLog:
        """Get a KPack build result log download URL.

        :param resource_group_name: The name of the resource group that contains the resource. You can
         obtain this value from the Azure Resource Manager API or the portal. Required.
        :type resource_group_name: str
        :param service_name: The name of the Service resource. Required.
        :type service_name: str
        :param build_service_name: The name of the build service resource. Required.
        :type build_service_name: str
        :param build_name: The name of the build resource. Required.
        :type build_name: str
        :param build_result_name: The name of the build result resource. Required.
        :type build_result_name: str
        :return: BuildResultLog or the result of cls(response)
        :rtype: ~azure.mgmt.appplatform.models.BuildResultLog
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map: MutableMapping[int, Type[HttpResponseError]] = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop("api_version", _params.pop("api-version", self._config.api_version))
        cls: ClsType[_models.BuildResultLog] = kwargs.pop("cls", None)

        _request = build_get_build_result_log_request(
            resource_group_name=resource_group_name,
            service_name=service_name,
            build_service_name=build_service_name,
            build_name=build_name,
            build_result_name=build_result_name,
            subscription_id=self._config.subscription_id,
            api_version=api_version,
            headers=_headers,
            params=_params,
        )
        _request.url = self._client.format_url(_request.url)

        _stream = False
        pipeline_response: PipelineResponse = self._client._pipeline.run(  # pylint: disable=protected-access
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response, error_format=ARMErrorFormat)

        deserialized = self._deserialize("BuildResultLog", pipeline_response.http_response)

        if cls:
            return cls(pipeline_response, deserialized, {})  # type: ignore

        return deserialized  # type: ignore

    @distributed_trace
    def get_resource_upload_url(
        self, resource_group_name: str, service_name: str, build_service_name: str, **kwargs: Any
    ) -> _models.ResourceUploadDefinition:
        """Get an resource upload URL for build service, which may be artifacts or source archive.

        :param resource_group_name: The name of the resource group that contains the resource. You can
         obtain this value from the Azure Resource Manager API or the portal. Required.
        :type resource_group_name: str
        :param service_name: The name of the Service resource. Required.
        :type service_name: str
        :param build_service_name: The name of the build service resource. Required.
        :type build_service_name: str
        :return: ResourceUploadDefinition or the result of cls(response)
        :rtype: ~azure.mgmt.appplatform.models.ResourceUploadDefinition
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map: MutableMapping[int, Type[HttpResponseError]] = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop("api_version", _params.pop("api-version", self._config.api_version))
        cls: ClsType[_models.ResourceUploadDefinition] = kwargs.pop("cls", None)

        _request = build_get_resource_upload_url_request(
            resource_group_name=resource_group_name,
            service_name=service_name,
            build_service_name=build_service_name,
            subscription_id=self._config.subscription_id,
            api_version=api_version,
            headers=_headers,
            params=_params,
        )
        _request.url = self._client.format_url(_request.url)

        _stream = False
        pipeline_response: PipelineResponse = self._client._pipeline.run(  # pylint: disable=protected-access
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response, error_format=ARMErrorFormat)

        deserialized = self._deserialize("ResourceUploadDefinition", pipeline_response.http_response)

        if cls:
            return cls(pipeline_response, deserialized, {})  # type: ignore

        return deserialized  # type: ignore

    @distributed_trace
    def list_supported_buildpacks(
        self, resource_group_name: str, service_name: str, build_service_name: str, **kwargs: Any
    ) -> _models.SupportedBuildpacksCollection:
        """Get all supported buildpacks.

        :param resource_group_name: The name of the resource group that contains the resource. You can
         obtain this value from the Azure Resource Manager API or the portal. Required.
        :type resource_group_name: str
        :param service_name: The name of the Service resource. Required.
        :type service_name: str
        :param build_service_name: The name of the build service resource. Required.
        :type build_service_name: str
        :return: SupportedBuildpacksCollection or the result of cls(response)
        :rtype: ~azure.mgmt.appplatform.models.SupportedBuildpacksCollection
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map: MutableMapping[int, Type[HttpResponseError]] = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop("api_version", _params.pop("api-version", self._config.api_version))
        cls: ClsType[_models.SupportedBuildpacksCollection] = kwargs.pop("cls", None)

        _request = build_list_supported_buildpacks_request(
            resource_group_name=resource_group_name,
            service_name=service_name,
            build_service_name=build_service_name,
            subscription_id=self._config.subscription_id,
            api_version=api_version,
            headers=_headers,
            params=_params,
        )
        _request.url = self._client.format_url(_request.url)

        _stream = False
        pipeline_response: PipelineResponse = self._client._pipeline.run(  # pylint: disable=protected-access
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response, error_format=ARMErrorFormat)

        deserialized = self._deserialize("SupportedBuildpacksCollection", pipeline_response.http_response)

        if cls:
            return cls(pipeline_response, deserialized, {})  # type: ignore

        return deserialized  # type: ignore

    @distributed_trace
    def get_supported_buildpack(
        self, resource_group_name: str, service_name: str, build_service_name: str, buildpack_name: str, **kwargs: Any
    ) -> _models.SupportedBuildpackResource:
        """Get the supported buildpack resource.

        :param resource_group_name: The name of the resource group that contains the resource. You can
         obtain this value from the Azure Resource Manager API or the portal. Required.
        :type resource_group_name: str
        :param service_name: The name of the Service resource. Required.
        :type service_name: str
        :param build_service_name: The name of the build service resource. Required.
        :type build_service_name: str
        :param buildpack_name: The name of the buildpack resource. Required.
        :type buildpack_name: str
        :return: SupportedBuildpackResource or the result of cls(response)
        :rtype: ~azure.mgmt.appplatform.models.SupportedBuildpackResource
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map: MutableMapping[int, Type[HttpResponseError]] = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop("api_version", _params.pop("api-version", self._config.api_version))
        cls: ClsType[_models.SupportedBuildpackResource] = kwargs.pop("cls", None)

        _request = build_get_supported_buildpack_request(
            resource_group_name=resource_group_name,
            service_name=service_name,
            build_service_name=build_service_name,
            buildpack_name=buildpack_name,
            subscription_id=self._config.subscription_id,
            api_version=api_version,
            headers=_headers,
            params=_params,
        )
        _request.url = self._client.format_url(_request.url)

        _stream = False
        pipeline_response: PipelineResponse = self._client._pipeline.run(  # pylint: disable=protected-access
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response, error_format=ARMErrorFormat)

        deserialized = self._deserialize("SupportedBuildpackResource", pipeline_response.http_response)

        if cls:
            return cls(pipeline_response, deserialized, {})  # type: ignore

        return deserialized  # type: ignore

    @distributed_trace
    def list_supported_stacks(
        self, resource_group_name: str, service_name: str, build_service_name: str, **kwargs: Any
    ) -> _models.SupportedStacksCollection:
        """Get all supported stacks.

        :param resource_group_name: The name of the resource group that contains the resource. You can
         obtain this value from the Azure Resource Manager API or the portal. Required.
        :type resource_group_name: str
        :param service_name: The name of the Service resource. Required.
        :type service_name: str
        :param build_service_name: The name of the build service resource. Required.
        :type build_service_name: str
        :return: SupportedStacksCollection or the result of cls(response)
        :rtype: ~azure.mgmt.appplatform.models.SupportedStacksCollection
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map: MutableMapping[int, Type[HttpResponseError]] = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop("api_version", _params.pop("api-version", self._config.api_version))
        cls: ClsType[_models.SupportedStacksCollection] = kwargs.pop("cls", None)

        _request = build_list_supported_stacks_request(
            resource_group_name=resource_group_name,
            service_name=service_name,
            build_service_name=build_service_name,
            subscription_id=self._config.subscription_id,
            api_version=api_version,
            headers=_headers,
            params=_params,
        )
        _request.url = self._client.format_url(_request.url)

        _stream = False
        pipeline_response: PipelineResponse = self._client._pipeline.run(  # pylint: disable=protected-access
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response, error_format=ARMErrorFormat)

        deserialized = self._deserialize("SupportedStacksCollection", pipeline_response.http_response)

        if cls:
            return cls(pipeline_response, deserialized, {})  # type: ignore

        return deserialized  # type: ignore

    @distributed_trace
    def get_supported_stack(
        self, resource_group_name: str, service_name: str, build_service_name: str, stack_name: str, **kwargs: Any
    ) -> _models.SupportedStackResource:
        """Get the supported stack resource.

        :param resource_group_name: The name of the resource group that contains the resource. You can
         obtain this value from the Azure Resource Manager API or the portal. Required.
        :type resource_group_name: str
        :param service_name: The name of the Service resource. Required.
        :type service_name: str
        :param build_service_name: The name of the build service resource. Required.
        :type build_service_name: str
        :param stack_name: The name of the stack resource. Required.
        :type stack_name: str
        :return: SupportedStackResource or the result of cls(response)
        :rtype: ~azure.mgmt.appplatform.models.SupportedStackResource
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map: MutableMapping[int, Type[HttpResponseError]] = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop("api_version", _params.pop("api-version", self._config.api_version))
        cls: ClsType[_models.SupportedStackResource] = kwargs.pop("cls", None)

        _request = build_get_supported_stack_request(
            resource_group_name=resource_group_name,
            service_name=service_name,
            build_service_name=build_service_name,
            stack_name=stack_name,
            subscription_id=self._config.subscription_id,
            api_version=api_version,
            headers=_headers,
            params=_params,
        )
        _request.url = self._client.format_url(_request.url)

        _stream = False
        pipeline_response: PipelineResponse = self._client._pipeline.run(  # pylint: disable=protected-access
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response, error_format=ARMErrorFormat)

        deserialized = self._deserialize("SupportedStackResource", pipeline_response.http_response)

        if cls:
            return cls(pipeline_response, deserialized, {})  # type: ignore

        return deserialized  # type: ignore



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
import pytest
from azure.mgmt.apimanagement import ApiManagementClient

from devtools_testutils import AzureMgmtRecordedTestCase, RandomNameResourceGroupPreparer, recorded_by_proxy

AZURE_LOCATION = "eastus"


@pytest.mark.skip("you may need to update the auto-generated test case before run it")
class TestApiManagementPrivateEndpointConnectionOperations(AzureMgmtRecordedTestCase):
    def setup_method(self, method):
        self.client = self.create_mgmt_client(ApiManagementClient)

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_private_endpoint_connection_list_by_service(self, resource_group):
        response = self.client.private_endpoint_connection.list_by_service(
            resource_group_name=resource_group.name,
            service_name="str",
            api_version="2024-05-01",
        )
        result = [r for r in response]
        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_private_endpoint_connection_get_by_name(self, resource_group):
        response = self.client.private_endpoint_connection.get_by_name(
            resource_group_name=resource_group.name,
            service_name="str",
            private_endpoint_connection_name="str",
            api_version="2024-05-01",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_private_endpoint_connection_begin_create_or_update(self, resource_group):
        response = self.client.private_endpoint_connection.begin_create_or_update(
            resource_group_name=resource_group.name,
            service_name="str",
            private_endpoint_connection_name="str",
            private_endpoint_connection_request={
                "id": "str",
                "properties": {
                    "privateLinkServiceConnectionState": {
                        "actionsRequired": "str",
                        "description": "str",
                        "status": "str",
                    }
                },
            },
            api_version="2024-05-01",
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_private_endpoint_connection_begin_delete(self, resource_group):
        response = self.client.private_endpoint_connection.begin_delete(
            resource_group_name=resource_group.name,
            service_name="str",
            private_endpoint_connection_name="str",
            api_version="2024-05-01",
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_private_endpoint_connection_list_private_link_resources(self, resource_group):
        response = self.client.private_endpoint_connection.list_private_link_resources(
            resource_group_name=resource_group.name,
            service_name="str",
            api_version="2024-05-01",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_private_endpoint_connection_get_private_link_resource(self, resource_group):
        response = self.client.private_endpoint_connection.get_private_link_resource(
            resource_group_name=resource_group.name,
            service_name="str",
            private_link_sub_resource_name="str",
            api_version="2024-05-01",
        )

        # please add some check logic here by yourself
        # ...



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from ._deny_assignments_operations import DenyAssignmentsOperations
from ._provider_operations_metadata_operations import ProviderOperationsMetadataOperations
from ._role_assignments_operations import RoleAssignmentsOperations
from ._permissions_operations import PermissionsOperations
from ._role_definitions_operations import RoleDefinitionsOperations

from ._patch import __all__ as _patch_all
from ._patch import *  # pylint: disable=unused-wildcard-import
from ._patch import patch_sdk as _patch_sdk

__all__ = [
    "DenyAssignmentsOperations",
    "ProviderOperationsMetadataOperations",
    "RoleAssignmentsOperations",
    "PermissionsOperations",
    "RoleDefinitionsOperations",
]
__all__.extend([p for p in _patch_all if p not in __all__])
_patch_sdk()



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from enum import Enum
from azure.core import CaseInsensitiveEnumMeta


class AccessIdName(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """AccessIdName."""

    ACCESS = "access"
    GIT_ACCESS = "gitAccess"


class AccessType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The type of access to be used for the storage account."""

    ACCESS_KEY = "AccessKey"
    """Use access key."""
    SYSTEM_ASSIGNED_MANAGED_IDENTITY = "SystemAssignedManagedIdentity"
    """Use system assigned managed identity."""
    USER_ASSIGNED_MANAGED_IDENTITY = "UserAssignedManagedIdentity"
    """Use user assigned managed identity."""


class AlwaysLog(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Specifies for what type of messages sampling settings should not apply."""

    ALL_ERRORS = "allErrors"
    """Always log all erroneous request regardless of sampling settings."""


class ApiGatewaySkuType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Name of the Sku."""

    STANDARD = "Standard"
    """Standard SKU of the API gateway."""
    WORKSPACE_GATEWAY_STANDARD = "WorkspaceGatewayStandard"
    """Standard SKU of the API gateway to be used in Workspaces."""
    WORKSPACE_GATEWAY_PREMIUM = "WorkspaceGatewayPremium"
    """Premium SKU of the API gateway to be used in Workspaces."""


class ApiManagementSkuCapacityScaleType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The scale type applicable to the sku."""

    AUTOMATIC = "Automatic"
    MANUAL = "Manual"
    NONE = "None"


class ApiManagementSkuRestrictionsReasonCode(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The reason for restriction."""

    QUOTA_ID = "QuotaId"
    NOT_AVAILABLE_FOR_SUBSCRIPTION = "NotAvailableForSubscription"


class ApiManagementSkuRestrictionsType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The type of restrictions."""

    LOCATION = "Location"
    ZONE = "Zone"


class ApimIdentityType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The type of identity used for the resource. The type 'SystemAssigned, UserAssigned' includes
    both an implicitly created identity and a set of user assigned identities. The type 'None' will
    remove any identities from the service.
    """

    SYSTEM_ASSIGNED = "SystemAssigned"
    USER_ASSIGNED = "UserAssigned"
    SYSTEM_ASSIGNED_USER_ASSIGNED = "SystemAssigned, UserAssigned"
    NONE = "None"


class ApiType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Type of API."""

    HTTP = "http"
    SOAP = "soap"
    WEBSOCKET = "websocket"
    GRAPHQL = "graphql"
    ODATA = "odata"
    GRPC = "grpc"


class AppType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """AppType."""

    PORTAL = "portal"
    """User create request was sent by legacy developer portal."""
    DEVELOPER_PORTAL = "developerPortal"
    """User create request was sent by new developer portal."""


class AsyncOperationStatus(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Status of an async operation."""

    STARTED = "Started"
    IN_PROGRESS = "InProgress"
    SUCCEEDED = "Succeeded"
    FAILED = "Failed"


class AsyncResolverStatus(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Status of an async resolver."""

    STARTED = "Started"
    IN_PROGRESS = "InProgress"
    SUCCEEDED = "Succeeded"
    FAILED = "Failed"


class AuthorizationMethod(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """AuthorizationMethod."""

    HEAD = "HEAD"
    OPTIONS = "OPTIONS"
    TRACE = "TRACE"
    GET = "GET"
    POST = "POST"
    PUT = "PUT"
    PATCH = "PATCH"
    DELETE = "DELETE"


class AuthorizationType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Authorization type options."""

    O_AUTH2 = "OAuth2"
    """OAuth2 authorization type"""


class BackendProtocol(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Backend communication protocol."""

    HTTP = "http"
    """The Backend is a RESTful service."""
    SOAP = "soap"
    """The Backend is a SOAP service."""


class BackendType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Type of the backend. A backend can be either Single or Pool."""

    SINGLE = "Single"
    """supports single backend"""
    POOL = "Pool"
    """supports pool backend"""


class BearerTokenSendingMethod(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """BearerTokenSendingMethod."""

    AUTHORIZATION_HEADER = "authorizationHeader"
    QUERY = "query"


class BearerTokenSendingMethods(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Form of an authorization grant, which the client uses to request the access token."""

    AUTHORIZATION_HEADER = "authorizationHeader"
    """Access token will be transmitted in the Authorization header using Bearer schema"""
    QUERY = "query"
    """Access token will be transmitted as query parameters."""


class CertificateConfigurationStoreName(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The System.Security.Cryptography.x509certificates.StoreName certificate store location. Only
    Root and CertificateAuthority are valid locations.
    """

    CERTIFICATE_AUTHORITY = "CertificateAuthority"
    ROOT = "Root"


class CertificateSource(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Certificate Source."""

    MANAGED = "Managed"
    KEY_VAULT = "KeyVault"
    CUSTOM = "Custom"
    BUILT_IN = "BuiltIn"


class CertificateStatus(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Certificate Status."""

    COMPLETED = "Completed"
    FAILED = "Failed"
    IN_PROGRESS = "InProgress"


class ClientAuthenticationMethod(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """ClientAuthenticationMethod."""

    BASIC = "Basic"
    """Basic Client Authentication method."""
    BODY = "Body"
    """Body based Authentication method."""


class ConfigurationIdName(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """ConfigurationIdName."""

    CONFIGURATION = "configuration"


class Confirmation(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Determines the type of confirmation e-mail that will be sent to the newly created user."""

    SIGNUP = "signup"
    """Send an e-mail to the user confirming they have successfully signed up."""
    INVITE = "invite"
    """Send an e-mail inviting the user to sign-up and complete registration."""


class ConnectionStatus(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The connection status."""

    UNKNOWN = "Unknown"
    CONNECTED = "Connected"
    DISCONNECTED = "Disconnected"
    DEGRADED = "Degraded"


class ConnectivityCheckProtocol(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The request's protocol. Specific protocol configuration can be available based on this
    selection. The specified destination address must be coherent with this value.
    """

    TCP = "TCP"
    HTTP = "HTTP"
    HTTPS = "HTTPS"


class ConnectivityStatusType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Resource Connectivity Status Type identifier."""

    INITIALIZING = "initializing"
    SUCCESS = "success"
    FAILURE = "failure"


class ContentFormat(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Format of the Content in which the API is getting imported. New formats can be added in the
    future.
    """

    WADL_XML = "wadl-xml"
    """The contents are inline and Content type is a WADL document."""
    WADL_LINK_JSON = "wadl-link-json"
    """The WADL document is hosted on a publicly accessible internet address."""
    SWAGGER_JSON = "swagger-json"
    """The contents are inline and Content Type is a OpenAPI 2.0 JSON Document."""
    SWAGGER_LINK_JSON = "swagger-link-json"
    """The OpenAPI 2.0 JSON document is hosted on a publicly accessible internet address."""
    WSDL = "wsdl"
    """The contents are inline and the document is a WSDL/Soap document."""
    WSDL_LINK = "wsdl-link"
    """The WSDL document is hosted on a publicly accessible internet address."""
    OPENAPI = "openapi"
    """The contents are inline and Content Type is a OpenAPI 3.0 YAML Document."""
    OPENAPI_JSON = "openapi+json"
    """The contents are inline and Content Type is a OpenAPI 3.0 JSON Document."""
    OPENAPI_LINK = "openapi-link"
    """The OpenAPI 3.0 YAML document is hosted on a publicly accessible internet address."""
    OPENAPI_JSON_LINK = "openapi+json-link"
    """The OpenAPI 3.0 JSON document is hosted on a publicly accessible internet address."""
    GRAPHQL_LINK = "graphql-link"
    """The GraphQL API endpoint hosted on a publicly accessible internet address."""
    ODATA = "odata"
    """The contents are inline and Content Type is a OData XML Document."""
    ODATA_LINK = "odata-link"
    """The OData metadata document hosted on a publicly accessible internet address."""
    GRPC = "grpc"
    """The contents are inline and Content Type is a gRPC protobuf file."""
    GRPC_LINK = "grpc-link"
    """The gRPC protobuf file is hosted on a publicly accessible internet address."""


class CreatedByType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The type of identity that created the resource."""

    USER = "User"
    APPLICATION = "Application"
    MANAGED_IDENTITY = "ManagedIdentity"
    KEY = "Key"


class DataMaskingMode(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Data masking mode."""

    MASK = "Mask"
    """Mask the value of an entity."""
    HIDE = "Hide"
    """Hide the presence of an entity."""


class DeveloperPortalStatus(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Status of developer portal in this API Management service."""

    ENABLED = "Enabled"
    """Developer Portal is enabled for the service."""
    DISABLED = "Disabled"
    """Developer Portal is disabled for the service."""


class ExportApi(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """ExportApi."""

    TRUE = "true"


class ExportFormat(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """ExportFormat."""

    SWAGGER = "swagger-link"
    """Export the Api Definition in OpenAPI 2.0 Specification as JSON document to the Storage Blob."""
    WSDL = "wsdl-link"
    """Export the Api Definition in WSDL Schema to Storage Blob. This is only supported for APIs of
    Type ``soap``"""
    WADL = "wadl-link"
    """Export the Api Definition in WADL Schema to Storage Blob."""
    OPENAPI = "openapi-link"
    """Export the Api Definition in OpenAPI 3.0 Specification as YAML document to Storage Blob."""
    OPENAPI_JSON = "openapi+json-link"
    """Export the Api Definition in OpenAPI 3.0 Specification as JSON document to Storage Blob."""


class ExportResultFormat(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Format in which the API Details are exported to the Storage Blob with Sas Key valid for 5
    minutes.
    """

    SWAGGER = "swagger-link-json"
    """The API Definition is exported in OpenAPI Specification 2.0 format to the Storage Blob."""
    WSDL = "wsdl-link+xml"
    """The API Definition is exported in WSDL Schema to Storage Blob. This is only supported for APIs
    of Type ``soap``"""
    WADL = "wadl-link-json"
    """Export the API Definition in WADL Schema to Storage Blob."""
    OPEN_API = "openapi-link"
    """Export the API Definition in OpenAPI Specification 3.0 to Storage Blob."""


class GatewayListDebugCredentialsContractPurpose(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Purpose of debug credential."""

    TRACING = "tracing"
    """The tracing purpose."""


class GatewaySkuCapacityScaleType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The scale type applicable to the sku."""

    AUTOMATIC = "Automatic"
    """Supported scale type automatic."""
    MANUAL = "Manual"
    """Supported scale type manual."""
    NONE = "None"
    """Scaling not supported."""


class GrantType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """GrantType."""

    AUTHORIZATION_CODE = "authorizationCode"
    """Authorization Code Grant flow as described https://tools.ietf.org/html/rfc6749#section-4.1."""
    IMPLICIT = "implicit"
    """Implicit Code Grant flow as described https://tools.ietf.org/html/rfc6749#section-4.2."""
    RESOURCE_OWNER_PASSWORD = "resourceOwnerPassword"
    """Resource Owner Password Grant flow as described
    https://tools.ietf.org/html/rfc6749#section-4.3."""
    CLIENT_CREDENTIALS = "clientCredentials"
    """Client Credentials Grant flow as described https://tools.ietf.org/html/rfc6749#section-4.4."""


class GroupType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Group type."""

    CUSTOM = "custom"
    SYSTEM = "system"
    EXTERNAL = "external"


class HostnameType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Hostname type."""

    PROXY = "Proxy"
    PORTAL = "Portal"
    MANAGEMENT = "Management"
    SCM = "Scm"
    DEVELOPER_PORTAL = "DeveloperPortal"
    CONFIGURATION_API = "ConfigurationApi"


class HttpCorrelationProtocol(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Sets correlation protocol to use for Application Insights diagnostics."""

    NONE = "None"
    """Do not read and inject correlation headers."""
    LEGACY = "Legacy"
    """Inject Request-Id and Request-Context headers with request correlation data. See
    https://github.com/dotnet/corefx/blob/master/src/System.Diagnostics.DiagnosticSource/src/HttpCorrelationProtocol.md."""
    W3_C = "W3C"
    """Inject Trace Context headers. See https://w3c.github.io/trace-context."""


class IdentityProviderType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """IdentityProviderType."""

    FACEBOOK = "facebook"
    """Facebook as Identity provider."""
    GOOGLE = "google"
    """Google as Identity provider."""
    MICROSOFT = "microsoft"
    """Microsoft Live as Identity provider."""
    TWITTER = "twitter"
    """Twitter as Identity provider."""
    AAD = "aad"
    """Azure Active Directory as Identity provider."""
    AAD_B2_C = "aadB2C"
    """Azure Active Directory B2C as Identity provider."""


class IssueType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The type of issue."""

    UNKNOWN = "Unknown"
    AGENT_STOPPED = "AgentStopped"
    GUEST_FIREWALL = "GuestFirewall"
    DNS_RESOLUTION = "DnsResolution"
    SOCKET_BIND = "SocketBind"
    NETWORK_SECURITY_RULE = "NetworkSecurityRule"
    USER_DEFINED_ROUTE = "UserDefinedRoute"
    PORT_THROTTLED = "PortThrottled"
    PLATFORM = "Platform"


class KeyType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The Key to be used to generate token for user."""

    PRIMARY = "primary"
    SECONDARY = "secondary"


class KeyVaultRefreshState(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """KeyVaultRefreshState."""

    TRUE = "true"
    """Entities for which KeyVault refresh failed."""
    FALSE = "false"
    """Entities for which KeyVault refresh succeeded"""


class LegacyApiState(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Indication whether or not the legacy Configuration API (v1) should be exposed on the API
    Management service. Value is optional but must be 'Enabled' or 'Disabled'. If 'Disabled',
    legacy Configuration API (v1) will not be available for self-hosted gateways. Default value is
    'Enabled'.
    """

    ENABLED = "Enabled"
    """Legacy Configuration API (v1) is enabled for the service and self-hosted gateways can connect
    to it."""
    DISABLED = "Disabled"
    """Legacy Configuration API (v1) is disabled for the service and self-hosted gateways can not
    connect to it."""


class LegacyPortalStatus(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Status of legacy portal in the API Management service."""

    ENABLED = "Enabled"
    """Legacy Portal is enabled for the service."""
    DISABLED = "Disabled"
    """Legacy Portal is disabled for the service."""


class LoggerType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Logger type."""

    AZURE_EVENT_HUB = "azureEventHub"
    """Azure Event Hub as log destination."""
    APPLICATION_INSIGHTS = "applicationInsights"
    """Azure Application Insights as log destination."""
    AZURE_MONITOR = "azureMonitor"
    """Azure Monitor"""


class Method(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The HTTP method to be used."""

    GET = "GET"
    POST = "POST"


class MigrateToStv2Mode(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Mode of Migration to stv2. Default is PreserveIp."""

    PRESERVE_IP = "PreserveIp"
    """Migrate API Management service to stv2 from stv1, by reserving the IP Address of the service.
    This will have a downtime of upto 15 minutes, while the IP address is getting migrate to new
    infrastructure."""
    NEW_IP = "NewIP"
    """Migrate API Management service to stv2 from stv1. This will have no downtime as the service
    configuration will be migrated to new infrastructure, but the IP address will changed."""


class NameAvailabilityReason(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Invalid indicates the name provided does not match the resource provider’s naming requirements
    (incorrect length, unsupported characters, etc.)  AlreadyExists indicates that the name is
    already in use and is therefore unavailable.
    """

    VALID = "Valid"
    INVALID = "Invalid"
    ALREADY_EXISTS = "AlreadyExists"


class NatGatewayState(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Property can be used to enable NAT Gateway for this API Management service."""

    ENABLED = "Enabled"
    """Nat Gateway is enabled for the service."""
    DISABLED = "Disabled"
    """Nat Gateway is disabled for the service."""


class NotificationName(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """NotificationName."""

    REQUEST_PUBLISHER_NOTIFICATION_MESSAGE = "RequestPublisherNotificationMessage"
    """The following email recipients and users will receive email notifications about subscription
    requests for API products requiring approval."""
    PURCHASE_PUBLISHER_NOTIFICATION_MESSAGE = "PurchasePublisherNotificationMessage"
    """The following email recipients and users will receive email notifications about new API product
    subscriptions."""
    NEW_APPLICATION_NOTIFICATION_MESSAGE = "NewApplicationNotificationMessage"
    """The following email recipients and users will receive email notifications when new applications
    are submitted to the application gallery."""
    BCC = "BCC"
    """The following recipients will receive blind carbon copies of all emails sent to developers."""
    NEW_ISSUE_PUBLISHER_NOTIFICATION_MESSAGE = "NewIssuePublisherNotificationMessage"
    """The following email recipients and users will receive email notifications when a new issue or
    comment is submitted on the developer portal."""
    ACCOUNT_CLOSED_PUBLISHER = "AccountClosedPublisher"
    """The following email recipients and users will receive email notifications when developer closes
    his account."""
    QUOTA_LIMIT_APPROACHING_PUBLISHER_NOTIFICATION_MESSAGE = "QuotaLimitApproachingPublisherNotificationMessage"
    """The following email recipients and users will receive email notifications when subscription
    usage gets close to usage quota."""


class OAuth2GrantType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """OAuth2 grant type options."""

    AUTHORIZATION_CODE = "AuthorizationCode"
    """Authorization Code grant"""
    CLIENT_CREDENTIALS = "ClientCredentials"
    """Client Credential grant"""


class OperationNameFormat(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The format of the Operation Name for Application Insights telemetries. Default is Name."""

    NAME = "Name"
    """API_NAME;rev=API_REVISION - OPERATION_NAME"""
    URL = "Url"
    """HTTP_VERB URL"""


class Origin(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The origin of the issue."""

    LOCAL = "Local"
    INBOUND = "Inbound"
    OUTBOUND = "Outbound"


class PlatformVersion(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Compute Platform Version running the service."""

    UNDETERMINED = "undetermined"
    """Platform version cannot be determined, as compute platform is not deployed."""
    STV1 = "stv1"
    """Platform running the service on Single Tenant V1 platform."""
    STV2 = "stv2"
    """Platform running the service on Single Tenant V2 platform."""
    MTV1 = "mtv1"
    """Platform running the service on Multi Tenant V1 platform."""
    STV2_1 = "stv2.1"
    """Platform running the service on Single Tenant V2 platform on newer Hardware."""


class PolicyComplianceState(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Policy Restriction Compliance State."""

    PENDING = "Pending"
    """The policy restriction compliance state has not yet been determined."""
    NON_COMPLIANT = "NonCompliant"
    """The scope in restriction is out of compliance."""
    COMPLIANT = "Compliant"
    """The scope in restriction is in compliance."""


class PolicyContentFormat(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Format of the policyContent."""

    XML = "xml"
    """The contents are inline and Content type is an XML document."""
    XML_LINK = "xml-link"
    """The policy XML document is hosted on a HTTP endpoint accessible from the API Management
    service."""
    RAWXML = "rawxml"
    """The contents are inline and Content type is a non XML encoded policy document."""
    RAWXML_LINK = "rawxml-link"
    """The policy document is not XML encoded and is hosted on a HTTP endpoint accessible from the API
    Management service."""


class PolicyExportFormat(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """PolicyExportFormat."""

    XML = "xml"
    """The contents are inline and Content type is an XML document."""
    RAWXML = "rawxml"
    """The contents are inline and Content type is a non XML encoded policy document."""


class PolicyFragmentContentFormat(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """PolicyFragmentContentFormat."""

    XML = "xml"
    """The contents are inline and Content type is an XML document."""
    RAWXML = "rawxml"
    """The contents are inline and Content type is a non XML encoded policy document."""


class PolicyIdName(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """PolicyIdName."""

    POLICY = "policy"


class PolicyRestrictionRequireBase(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Indicates if base policy should be enforced for the policy document."""

    TRUE = "true"
    """The policy is required to have base policy"""
    FALSE = "false"
    """The policy does not require to have base policy"""


class PolicyScopeContract(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """PolicyScopeContract."""

    TENANT = "Tenant"
    PRODUCT = "Product"
    API = "Api"
    OPERATION = "Operation"
    ALL = "All"


class PortalRevisionStatus(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Status of the portal's revision."""

    PENDING = "pending"
    """Portal's revision has been queued."""
    PUBLISHING = "publishing"
    """Portal's revision is being published."""
    COMPLETED = "completed"
    """Portal's revision publishing completed."""
    FAILED = "failed"
    """Portal's revision publishing failed."""


class PortalSettingsCspMode(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The mode of the developer portal Content Security Policy (CSP)."""

    ENABLED = "enabled"
    """The browser will block requests not matching allowed origins."""
    DISABLED = "disabled"
    """The browser will not apply the origin restrictions."""
    REPORT_ONLY = "reportOnly"
    """The browser will report requests not matching allowed origins without blocking them."""


class PreferredIPVersion(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The IP version to be used. Only IPv4 is supported for now."""

    I_PV4 = "IPv4"


class PrivateEndpointConnectionProvisioningState(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The current provisioning state."""

    SUCCEEDED = "Succeeded"
    CREATING = "Creating"
    DELETING = "Deleting"
    FAILED = "Failed"


class PrivateEndpointServiceConnectionStatus(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The private endpoint connection status."""

    PENDING = "Pending"
    APPROVED = "Approved"
    REJECTED = "Rejected"


class ProductState(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """whether product is published or not. Published products are discoverable by users of developer
    portal. Non published products are visible only to administrators. Default state of Product is
    notPublished.
    """

    NOT_PUBLISHED = "notPublished"
    PUBLISHED = "published"


class Protocol(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Protocol."""

    HTTP = "http"
    HTTPS = "https"
    WS = "ws"
    WSS = "wss"


class PublicNetworkAccess(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Whether or not public endpoint access is allowed for this API Management service.  Value is
    optional but if passed in, must be 'Enabled' or 'Disabled'. If 'Disabled', private endpoints
    are the exclusive access method. Default value is 'Enabled'.
    """

    ENABLED = "Enabled"
    DISABLED = "Disabled"


class ResourceSkuCapacityScaleType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The scale type applicable to the sku."""

    AUTOMATIC = "automatic"
    """Supported scale type automatic."""
    MANUAL = "manual"
    """Supported scale type manual."""
    NONE = "none"
    """Scaling not supported."""


class SamplingType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Sampling type."""

    FIXED = "fixed"
    """Fixed-rate sampling."""


class SchemaType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Schema Type. Immutable."""

    XML = "xml"
    """XML schema type."""
    JSON = "json"
    """Json schema type."""


class SettingsTypeName(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """SettingsTypeName."""

    PUBLIC = "public"


class Severity(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The severity of the issue."""

    ERROR = "Error"
    WARNING = "Warning"


class SkuType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Name of the Sku."""

    DEVELOPER = "Developer"
    """Developer SKU of Api Management."""
    STANDARD = "Standard"
    """Standard SKU of Api Management."""
    PREMIUM = "Premium"
    """Premium SKU of Api Management."""
    BASIC = "Basic"
    """Basic SKU of Api Management."""
    CONSUMPTION = "Consumption"
    """Consumption SKU of Api Management."""
    ISOLATED = "Isolated"
    """Isolated SKU of Api Management."""
    BASIC_V2 = "BasicV2"
    """BasicV2 SKU of Api Management."""
    STANDARD_V2 = "StandardV2"
    """StandardV2 SKU of Api Management."""


class SoapApiType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Type of API to create.


    * ``http`` creates a REST API
    * ``soap`` creates a SOAP pass-through API
    * ``websocket`` creates websocket API
    * ``graphql`` creates GraphQL API.
      New types can be added in the future.
    """

    SOAP_TO_REST = "http"
    """Imports a SOAP API having a RESTful front end."""
    SOAP_PASS_THROUGH = "soap"
    """Imports the SOAP API having a SOAP front end."""
    WEB_SOCKET = "websocket"
    """Imports the API having a Websocket front end."""
    GRAPH_QL = "graphql"
    """Imports the API having a GraphQL front end."""
    O_DATA = "odata"
    """Imports the API having a OData front end."""
    G_RPC = "grpc"
    """Imports the API having a gRPC front end."""


class State(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Status of the issue."""

    PROPOSED = "proposed"
    """The issue is proposed."""
    OPEN = "open"
    """The issue is opened."""
    REMOVED = "removed"
    """The issue was removed."""
    RESOLVED = "resolved"
    """The issue is now resolved."""
    CLOSED = "closed"
    """The issue was closed."""


class SubscriptionState(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Subscription state. Possible states are * active – the subscription is active, * suspended –
    the subscription is blocked, and the subscriber cannot call any APIs of the product, *
    submitted – the subscription request has been made by the developer, but has not yet been
    approved or rejected, * rejected – the subscription request has been denied by an
    administrator, * cancelled – the subscription has been cancelled by the developer or
    administrator, * expired – the subscription reached its expiration date and was deactivated.
    """

    SUSPENDED = "suspended"
    ACTIVE = "active"
    EXPIRED = "expired"
    SUBMITTED = "submitted"
    REJECTED = "rejected"
    CANCELLED = "cancelled"


class TemplateName(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """TemplateName."""

    APPLICATION_APPROVED_NOTIFICATION_MESSAGE = "applicationApprovedNotificationMessage"
    ACCOUNT_CLOSED_DEVELOPER = "accountClosedDeveloper"
    QUOTA_LIMIT_APPROACHING_DEVELOPER_NOTIFICATION_MESSAGE = "quotaLimitApproachingDeveloperNotificationMessage"
    NEW_DEVELOPER_NOTIFICATION_MESSAGE = "newDeveloperNotificationMessage"
    EMAIL_CHANGE_IDENTITY_DEFAULT = "emailChangeIdentityDefault"
    INVITE_USER_NOTIFICATION_MESSAGE = "inviteUserNotificationMessage"
    NEW_COMMENT_NOTIFICATION_MESSAGE = "newCommentNotificationMessage"
    CONFIRM_SIGN_UP_IDENTITY_DEFAULT = "confirmSignUpIdentityDefault"
    NEW_ISSUE_NOTIFICATION_MESSAGE = "newIssueNotificationMessage"
    PURCHASE_DEVELOPER_NOTIFICATION_MESSAGE = "purchaseDeveloperNotificationMessage"
    PASSWORD_RESET_IDENTITY_DEFAULT = "passwordResetIdentityDefault"
    PASSWORD_RESET_BY_ADMIN_NOTIFICATION_MESSAGE = "passwordResetByAdminNotificationMessage"
    REJECT_DEVELOPER_NOTIFICATION_MESSAGE = "rejectDeveloperNotificationMessage"
    REQUEST_DEVELOPER_NOTIFICATION_MESSAGE = "requestDeveloperNotificationMessage"


class TranslateRequiredQueryParametersConduct(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Strategy of translating required query parameters to template ones. By default has value
    'template'. Possible values: 'template', 'query'.
    """

    TEMPLATE = "template"
    """Translates required query parameters to template ones. Is a default value"""
    QUERY = "query"
    """Leaves required query parameters as they are (no translation done)."""


class UserState(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Account state. Specifies whether the user is active or not. Blocked users are unable to sign
    into the developer portal or call any APIs of subscribed products. Default state is Active.
    """

    ACTIVE = "active"
    """User state is active."""
    BLOCKED = "blocked"
    """User is blocked. Blocked users cannot authenticate at developer portal or call API."""
    PENDING = "pending"
    """User account is pending. Requires identity confirmation before it can be made active."""
    DELETED = "deleted"
    """User account is closed. All identities and related entities are removed."""


class Verbosity(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The verbosity level applied to traces emitted by trace policies."""

    VERBOSE = "verbose"
    """All the traces emitted by trace policies will be sent to the logger attached to this diagnostic
    instance."""
    INFORMATION = "information"
    """Traces with 'severity' set to 'information' and 'error' will be sent to the logger attached to
    this diagnostic instance."""
    ERROR = "error"
    """Only traces with 'severity' set to 'error' will be sent to the logger attached to this
    diagnostic instance."""


class VersioningScheme(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """An value that determines where the API Version identifier will be located in a HTTP request."""

    SEGMENT = "Segment"
    """The API Version is passed in a path segment."""
    QUERY = "Query"
    """The API Version is passed in a query parameter."""
    HEADER = "Header"
    """The API Version is passed in a HTTP header."""


class VirtualNetworkType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The type of VPN in which API Management gateway needs to be configured in."""

    NONE = "None"
    """The API Management gateway is not part of any Virtual Network."""
    EXTERNAL = "External"
    """The API Management gateway is part of Virtual Network and it is accessible from Internet."""
    INTERNAL = "Internal"
    """The API Management gateway is part of Virtual Network and it is only accessible from within the
    virtual network."""



# pylint: disable=too-many-lines,too-many-statements
# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
import sys
from typing import Any, AsyncIterable, Callable, Dict, Optional, Type, TypeVar
import urllib.parse

from azure.core.async_paging import AsyncItemPaged, AsyncList
from azure.core.exceptions import (
    ClientAuthenticationError,
    HttpResponseError,
    ResourceExistsError,
    ResourceNotFoundError,
    ResourceNotModifiedError,
    map_error,
)
from azure.core.pipeline import PipelineResponse
from azure.core.rest import AsyncHttpResponse, HttpRequest
from azure.core.tracing.decorator import distributed_trace
from azure.core.tracing.decorator_async import distributed_trace_async
from azure.core.utils import case_insensitive_dict
from azure.mgmt.core.exceptions import ARMErrorFormat

from ... import models as _models
from ...operations._departments_operations import build_get_request, build_list_by_billing_account_request

if sys.version_info >= (3, 9):
    from collections.abc import MutableMapping
else:
    from typing import MutableMapping  # type: ignore  # pylint: disable=ungrouped-imports
T = TypeVar("T")
ClsType = Optional[Callable[[PipelineResponse[HttpRequest, AsyncHttpResponse], T, Dict[str, Any]], Any]]


class DepartmentsOperations:
    """
    .. warning::
        **DO NOT** instantiate this class directly.

        Instead, you should access the following operations through
        :class:`~azure.mgmt.billing.aio.BillingManagementClient`'s
        :attr:`departments` attribute.
    """

    models = _models

    def __init__(self, *args, **kwargs) -> None:
        input_args = list(args)
        self._client = input_args.pop(0) if input_args else kwargs.pop("client")
        self._config = input_args.pop(0) if input_args else kwargs.pop("config")
        self._serialize = input_args.pop(0) if input_args else kwargs.pop("serializer")
        self._deserialize = input_args.pop(0) if input_args else kwargs.pop("deserializer")

    @distributed_trace_async
    async def get(self, billing_account_name: str, department_name: str, **kwargs: Any) -> _models.Department:
        """Gets a department by ID. The operation is supported only for billing accounts with agreement
        type Enterprise Agreement.

        .. seealso::
           - https://docs.microsoft.com/en-us/rest/api/billing/

        :param billing_account_name: The ID that uniquely identifies a billing account. Required.
        :type billing_account_name: str
        :param department_name: The name of the department. Required.
        :type department_name: str
        :return: Department or the result of cls(response)
        :rtype: ~azure.mgmt.billing.models.Department
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map: MutableMapping[int, Type[HttpResponseError]] = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop("api_version", _params.pop("api-version", self._config.api_version))
        cls: ClsType[_models.Department] = kwargs.pop("cls", None)

        _request = build_get_request(
            billing_account_name=billing_account_name,
            department_name=department_name,
            api_version=api_version,
            headers=_headers,
            params=_params,
        )
        _request.url = self._client.format_url(_request.url)

        _stream = False
        pipeline_response: PipelineResponse = await self._client._pipeline.run(  # pylint: disable=protected-access
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            error = self._deserialize.failsafe_deserialize(_models.ErrorResponse, pipeline_response)
            raise HttpResponseError(response=response, model=error, error_format=ARMErrorFormat)

        deserialized = self._deserialize("Department", pipeline_response.http_response)

        if cls:
            return cls(pipeline_response, deserialized, {})  # type: ignore

        return deserialized  # type: ignore

    @distributed_trace
    def list_by_billing_account(
        self,
        billing_account_name: str,
        filter: Optional[str] = None,
        order_by: Optional[str] = None,
        top: Optional[int] = None,
        skip: Optional[int] = None,
        search: Optional[str] = None,
        **kwargs: Any
    ) -> AsyncIterable["_models.Department"]:
        """Lists the departments that a user has access to. The operation is supported only for billing
        accounts with agreement type Enterprise Agreement.

        .. seealso::
           - https://docs.microsoft.com/en-us/rest/api/billing/

        :param billing_account_name: The ID that uniquely identifies a billing account. Required.
        :type billing_account_name: str
        :param filter: The filter query option allows clients to filter a collection of resources that
         are addressed by a request URL. Default value is None.
        :type filter: str
        :param order_by: The orderby query option allows clients to request resources in a particular
         order. Default value is None.
        :type order_by: str
        :param top: The top query option requests the number of items in the queried collection to be
         included in the result. The maximum supported value for top is 50. Default value is None.
        :type top: int
        :param skip: The skip query option requests the number of items in the queried collection that
         are to be skipped and not included in the result. Default value is None.
        :type skip: int
        :param search: The search query option allows clients to request items within a collection
         matching a free-text search expression. search is only supported for string fields. Default
         value is None.
        :type search: str
        :return: An iterator like instance of either Department or the result of cls(response)
        :rtype: ~azure.core.async_paging.AsyncItemPaged[~azure.mgmt.billing.models.Department]
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        _headers = kwargs.pop("headers", {}) or {}
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop("api_version", _params.pop("api-version", self._config.api_version))
        cls: ClsType[_models.DepartmentListResult] = kwargs.pop("cls", None)

        error_map: MutableMapping[int, Type[HttpResponseError]] = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        def prepare_request(next_link=None):
            if not next_link:

                _request = build_list_by_billing_account_request(
                    billing_account_name=billing_account_name,
                    filter=filter,
                    order_by=order_by,
                    top=top,
                    skip=skip,
                    search=search,
                    api_version=api_version,
                    headers=_headers,
                    params=_params,
                )
                _request.url = self._client.format_url(_request.url)

            else:
                # make call to next link with the client's api-version
                _parsed_next_link = urllib.parse.urlparse(next_link)
                _next_request_params = case_insensitive_dict(
                    {
                        key: [urllib.parse.quote(v) for v in value]
                        for key, value in urllib.parse.parse_qs(_parsed_next_link.query).items()
                    }
                )
                _next_request_params["api-version"] = self._config.api_version
                _request = HttpRequest(
                    "GET", urllib.parse.urljoin(next_link, _parsed_next_link.path), params=_next_request_params
                )
                _request.url = self._client.format_url(_request.url)
                _request.method = "GET"
            return _request

        async def extract_data(pipeline_response):
            deserialized = self._deserialize("DepartmentListResult", pipeline_response)
            list_of_elem = deserialized.value
            if cls:
                list_of_elem = cls(list_of_elem)  # type: ignore
            return deserialized.next_link or None, AsyncList(list_of_elem)

        async def get_next(next_link=None):
            _request = prepare_request(next_link)

            _stream = False
            pipeline_response: PipelineResponse = await self._client._pipeline.run(  # pylint: disable=protected-access
                _request, stream=_stream, **kwargs
            )
            response = pipeline_response.http_response

            if response.status_code not in [200]:
                map_error(status_code=response.status_code, response=response, error_map=error_map)
                error = self._deserialize.failsafe_deserialize(_models.ErrorResponse, pipeline_response)
                raise HttpResponseError(response=response, model=error, error_format=ARMErrorFormat)

            return pipeline_response

        return AsyncItemPaged(get_next, extract_data)



import pytest
from devtools_testutils import test_proxy, remove_batch_sanitizers


# autouse=True will trigger this fixture on each pytest run, even if it's not explicitly used by a test method
@pytest.fixture(scope="session", autouse=True)
def start_proxy(test_proxy):
    # Remove the following sanitizers since certain fields are needed in tests and are non-sensitive:
    #  - AZSDK3430: $..id
    #  - AZSDK3493: $..name
    remove_batch_sanitizers(["AZSDK3430", "AZSDK3493"])
    return



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for
# license information.
#
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is
# regenerated.
# --------------------------------------------------------------------------

from msrest.serialization import Model


class Ipv4Config(Model):
    """Details related to the IPv4 address configuration.

    Variables are only populated by the server, and will be ignored when
    sending a request.

    :ivar ip_address: The IPv4 address of the network adapter.
    :vartype ip_address: str
    :ivar subnet: The IPv4 subnet of the network adapter.
    :vartype subnet: str
    :ivar gateway: The IPv4 gateway of the network adapter.
    :vartype gateway: str
    """

    _validation = {
        'ip_address': {'readonly': True},
        'subnet': {'readonly': True},
        'gateway': {'readonly': True},
    }

    _attribute_map = {
        'ip_address': {'key': 'ipAddress', 'type': 'str'},
        'subnet': {'key': 'subnet', 'type': 'str'},
        'gateway': {'key': 'gateway', 'type': 'str'},
    }

    def __init__(self, **kwargs) -> None:
        super(Ipv4Config, self).__init__(**kwargs)
        self.ip_address = None
        self.subnet = None
        self.gateway = None



# --------------------------------------------------------------------------
#
# Copyright (c) Microsoft Corporation. All rights reserved.
#
# The MIT License (MIT)
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the ""Software""), to
# deal in the Software without restriction, including without limitation the
# rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
# sell copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED *AS IS*, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
# IN THE SOFTWARE.
#
# --------------------------------------------------------------------------
import os
import sys

import pytest

from devtools_testutils import (
    test_proxy,
    add_general_regex_sanitizer,
    add_header_regex_sanitizer,
    set_default_session_settings,
    add_uri_regex_sanitizer,
    add_body_key_sanitizer,
    remove_batch_sanitizers,
)
from router_test_constants import SANITIZED, FAKE_FUNCTION_URI, FAKE_ENDPOINT, FAKE_CONNECTION_STRING
from azure.communication.jobrouter._shared.utils import parse_connection_str


# fixture needs to be visible from conftest


# autouse=True will trigger this fixture on each pytest run, even if it's not explicitly used by a test method
@pytest.fixture(scope="session", autouse=True)
def start_proxy(test_proxy):
    set_default_session_settings()

    communication_connection_string = os.getenv(
        "COMMUNICATION_LIVETEST_DYNAMIC_CONNECTION_STRING", FAKE_CONNECTION_STRING
    )

    add_general_regex_sanitizer(regex=communication_connection_string, value=FAKE_CONNECTION_STRING)

    endpoint, _ = parse_connection_str(communication_connection_string)
    add_general_regex_sanitizer(regex=endpoint, value=FAKE_ENDPOINT)

    add_header_regex_sanitizer(key="repeatability-first-sent", value=SANITIZED)
    add_header_regex_sanitizer(key="repeatability-request-id", value=SANITIZED)
    add_header_regex_sanitizer(key="x-ms-content-sha256", value=SANITIZED)
    add_header_regex_sanitizer(key="etag", value=SANITIZED)

    add_body_key_sanitizer(json_path="$..access_token", value="access_token")
    add_body_key_sanitizer(json_path="$..primaryKey", value="primaryKey")
    add_body_key_sanitizer(json_path="$..secondaryKey", value="secondaryKey")
    add_body_key_sanitizer(json_path="$..etag", value=SANITIZED)
    add_body_key_sanitizer(json_path="$..functionUri", value=FAKE_FUNCTION_URI)
    add_body_key_sanitizer(json_path="$..functionKey", value=SANITIZED)
    add_body_key_sanitizer(json_path="$..appKey", value=SANITIZED)
    add_body_key_sanitizer(json_path="$..clientId", value=SANITIZED)

    # Remove the following sanitizers since certain fields are needed in tests and are non-sensitive:
    #  - AZSDK3430: $..id
    #  - AZSDK3493: $..name
    remove_batch_sanitizers(["AZSDK3430", "AZSDK3493"])
    return


@pytest.fixture(scope="function", autouse=True)
def initialize_test(request):
    if request.node.originalname is None:
        # tox throws error otherwise
        test_name = os.environ.get("PYTEST_CURRENT_TEST").split(":")[-1].split(" ")[0]
        request.cls._testMethodName = test_name
    else:
        request.cls._testMethodName = request.node.originalname


@pytest.fixture(scope="class", autouse=True)
def initialize_class(request):
    request.cls.queue_ids = {}  # type: Dict[str, List[str]]
    request.cls.distribution_policy_ids = {}  # type: Dict[str, List[str]]
    request.cls.exception_policy_ids = {}  # type: Dict[str, List[str]]
    request.cls.classification_policy_ids = {}  # type: Dict[str, List[str]]
    request.cls.worker_ids = {}  # type: Dict[str, List[str]]
    request.cls.job_ids = {}  # type: Dict[str, List[str]]


# Ignore async tests for Python < 3.5
collect_ignore_glob = []
if sys.version_info < (3, 5):
    collect_ignore_glob.append("*_async.py")



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from azure.identity import DefaultAzureCredential
from azure.mgmt.dashboard import DashboardManagementClient

"""
# PREREQUISITES
    pip install azure-identity
    pip install azure-mgmt-dashboard
# USAGE
    python grafana_update.py

    Before run the sample, please set the values of the client ID, tenant ID and client secret
    of the AAD application as environment variables: AZURE_CLIENT_ID, AZURE_TENANT_ID,
    AZURE_CLIENT_SECRET. For more info about how to get the value, please see:
    https://docs.microsoft.com/azure/active-directory/develop/howto-create-service-principal-portal
"""


def main():
    client = DashboardManagementClient(
        credential=DefaultAzureCredential(),
        subscription_id="00000000-0000-0000-0000-000000000000",
    )

    response = client.grafana.update(
        resource_group_name="myResourceGroup",
        workspace_name="myWorkspace",
        request_body_parameters={
            "properties": {
                "apiKey": "Enabled",
                "deterministicOutboundIP": "Enabled",
                "enterpriseConfigurations": {"marketplaceAutoRenew": "Enabled", "marketplacePlanId": "myPlanId"},
                "grafanaConfigurations": {
                    "smtp": {
                        "enabled": True,
                        "fromAddress": "test@sendemail.com",
                        "fromName": "emailsender",
                        "host": "smtp.sendemail.com:587",
                        "password": "<password>",
                        "skipVerify": True,
                        "startTLSPolicy": "OpportunisticStartTLS",
                        "user": "username",
                    }
                },
                "grafanaIntegrations": {
                    "azureMonitorWorkspaceIntegrations": [
                        {
                            "azureMonitorWorkspaceResourceId": "/subscriptions/00000000-0000-0000-0000-000000000000/resourcegroups/myResourceGroup/providers/microsoft.monitor/accounts/myAzureMonitorWorkspace"
                        }
                    ]
                },
                "grafanaMajorVersion": "9",
                "grafanaPlugins": {"sample-plugin-id": {}},
            },
            "sku": {"name": "Standard"},
            "tags": {"Environment": "Dev 2"},
        },
    )
    print(response)


# x-ms-original-file: specification/dashboard/resource-manager/Microsoft.Dashboard/stable/2023-09-01/examples/Grafana_Update.json
if __name__ == "__main__":
    main()



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import os
import uuid
import string
import random

from devtools_testutils.perfstress_tests import PerfStressTest

from azure.core import PipelineClient, AsyncPipelineClient
from azure.core.pipeline import Pipeline, AsyncPipeline
from azure.core.pipeline.transport import (
    RequestsTransport,
    AioHttpTransport,
    AsyncioRequestsTransport,
)
from azure.core.pipeline.policies import (
    UserAgentPolicy,
    HeadersPolicy,
    ProxyPolicy,
    NetworkTraceLoggingPolicy,
    HttpLoggingPolicy,
    RetryPolicy,
    CustomHookPolicy,
    RedirectPolicy,
    AsyncRetryPolicy,
    AsyncRedirectPolicy,
    BearerTokenCredentialPolicy,
    AsyncBearerTokenCredentialPolicy,
)
import azure.core.pipeline.policies as policies
from azure.core.credentials import AzureNamedKeyCredential
from azure.core.exceptions import (
    ClientAuthenticationError,
    ResourceExistsError,
    ResourceNotFoundError,
    ResourceNotModifiedError,
)
from azure.identity import ClientSecretCredential
from azure.identity.aio import ClientSecretCredential as AsyncClientSecretCredential
from azure.data.tables.aio import TableClient

from azure.storage.blob._shared.authentication import SharedKeyCredentialPolicy as BlobSharedKeyCredentialPolicy
from azure.data.tables._authentication import SharedKeyCredentialPolicy as TableSharedKeyCredentialPolicy

_LETTERS = string.ascii_letters


class _ServiceTest(PerfStressTest):
    transport = None
    async_transport = None

    def __init__(self, arguments):
        super().__init__(arguments)
        self.account_name = self.get_from_env("AZURE_STORAGE_ACCOUNT_NAME")
        self.account_key = self.get_from_env("AZURE_STORAGE_ACCOUNT_KEY")
        async_transport_types = {"aiohttp": AioHttpTransport, "requests": AsyncioRequestsTransport}
        sync_transport_types = {"requests": RequestsTransport}
        self.tenant_id = os.environ["AZURE-CORE_TENANT_ID"]
        self.client_id = os.environ["AZURE-CORE_CLIENT_ID"]
        self.client_secret = os.environ["AZURE-CORE_CLIENT_SECRET"]
        self.storage_scope = "https://storage.azure.com/.default"

        # defaults transports
        self.sync_transport = RequestsTransport
        self.async_transport = AioHttpTransport

        # if transport is specified, use that
        if self.args.transport:
            # if sync, override sync default
            if self.args.sync:
                try:
                    self.sync_transport = sync_transport_types[self.args.transport]
                except KeyError:
                    raise ValueError(f"Invalid sync transport:{self.args.transport}\n Valid options are:\n- requests\n")
            # if async, override async default
            else:
                try:
                    self.async_transport = async_transport_types[self.args.transport]
                except KeyError:
                    raise ValueError(
                        f"Invalid async transport:{self.args.transport}\n Valid options are:\n- aiohttp\n- requests\n"
                    )

        self.error_map = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }

    def _build_sync_pipeline_client(self, auth_policy):
        default_policies = [
            UserAgentPolicy,
            HeadersPolicy,
            ProxyPolicy,
            NetworkTraceLoggingPolicy,
            HttpLoggingPolicy,
            RetryPolicy,
            CustomHookPolicy,
            RedirectPolicy,
        ]

        if self.args.policies is None:
            # if None, only auth policy is passed in
            sync_pipeline = Pipeline(transport=self.sync_transport(), policies=[auth_policy])
        elif self.args.policies == "all":
            # if all, autorest default policies + auth policy
            sync_policies = [auth_policy]
            sync_policies.extend([policy(sdk_moniker=self.sdk_moniker) for policy in default_policies])
            sync_pipeline = Pipeline(transport=self.sync_transport(), policies=sync_policies)
        else:
            sync_policies = [auth_policy]
            for p in self.args.policies.split(","):
                try:
                    policy = getattr(policies, p)
                except AttributeError as exc:
                    raise ValueError(
                        f"Azure Core has no policy named {exc.name}. Please use policies from the following list: {policies.__all__}"
                    ) from exc
                sync_policies.append(policy(sdk_moniker=self.sdk_moniker))
            sync_pipeline = Pipeline(transport=self.sync_transport(), policies=sync_policies)
        return PipelineClient(self.account_endpoint, pipeline=sync_pipeline)

    def _build_async_pipeline_client(self, auth_policy):
        default_policies = [
            UserAgentPolicy,
            HeadersPolicy,
            ProxyPolicy,
            NetworkTraceLoggingPolicy,
            HttpLoggingPolicy,
            AsyncRetryPolicy,
            CustomHookPolicy,
            AsyncRedirectPolicy,
        ]
        if self.args.policies is None:
            # if None, only auth policy is passed in
            async_pipeline = AsyncPipeline(transport=self.async_transport(), policies=[auth_policy])
        elif self.args.policies == "all":
            # if all, autorest default policies + auth policy
            async_policies = [auth_policy]
            async_policies.extend([policy(sdk_moniker=self.sdk_moniker) for policy in default_policies])
            async_pipeline = AsyncPipeline(transport=self.async_transport(), policies=async_policies)
        else:
            async_policies = [auth_policy]
            # if custom list of policies, pass in custom list + auth policy
            for p in self.args.policies.split(","):
                try:
                    policy = getattr(policies, p)
                except AttributeError as exc:
                    raise ValueError(
                        f"Azure Core has no policy named {exc.name}. Please use policies from the following list: {policies.__all__}"
                    ) from exc
                async_policies.append(policy(sdk_moniker=self.sdk_moniker))
            async_pipeline = AsyncPipeline(transport=self.async_transport(), policies=async_policies)
        return AsyncPipelineClient(self.account_endpoint, pipeline=async_pipeline)

    def _set_auth_policies(self):
        if not self.args.use_entra_id:
            # if tables, create table credential policy, else blob policy
            if "tables" in self.sdk_moniker:
                self.sync_auth_policy = TableSharedKeyCredentialPolicy(
                    AzureNamedKeyCredential(self.account_name, self.account_key)
                )
                self.async_auth_policy = self.sync_auth_policy
            else:
                self.sync_auth_policy = BlobSharedKeyCredentialPolicy(self.account_name, self.account_key)
                self.async_auth_policy = self.sync_auth_policy
        else:
            sync_credential = ClientSecretCredential(self.tenant_id, self.client_id, self.client_secret)
            self.sync_auth_policy = BearerTokenCredentialPolicy(sync_credential, self.storage_scope)
            async_credential = AsyncClientSecretCredential(self.tenant_id, self.client_id, self.client_secret)
            self.async_auth_policy = AsyncBearerTokenCredentialPolicy(async_credential, self.storage_scope)

    @staticmethod
    def add_arguments(parser):
        super(_ServiceTest, _ServiceTest).add_arguments(parser)
        parser.add_argument(
            "--transport",
            nargs="?",
            type=str,
            help="""Underlying HttpTransport type. Defaults to `aiohttp` if async, `requests` if sync. Other possible values for async:\n"""
            """ - `requests`\n""",
            default=None,
        )
        parser.add_argument(
            "-s", "--size", nargs="?", type=int, help="Size of data to transfer.  Default is 10240.", default=10240
        )
        parser.add_argument(
            "--policies",
            nargs="?",
            type=str,
            help="""List of policies to pass in to the pipeline. Options:"""
            """\n- None: No extra policies passed in, except for authentication policy. This is the default."""
            """\n- 'all': All policies added automatically by autorest."""
            """\n- 'policy1,policy2': Comma-separated list of policies, such as 'RetryPolicy,HttpLoggingPolicy'""",
            default=None,
        )
        parser.add_argument(
            "--use-entra-id", action="store_true", help="Use Microsoft Entra ID authentication instead of shared key."
        )


class _BlobTest(_ServiceTest):
    container_name = "perfstress-" + str(uuid.uuid4())

    def __init__(self, arguments):
        super().__init__(arguments)
        self.account_endpoint = self.get_from_env("AZURE_STORAGE_BLOBS_ENDPOINT")
        self.container_name = self.get_from_env("AZURE_STORAGE_CONTAINER_NAME")
        self.api_version = "2021-12-02"
        self.sdk_moniker = f"storage-blob/{self.api_version}"

        self._set_auth_policies()
        self.pipeline_client = self._build_sync_pipeline_client(self.sync_auth_policy)
        self.async_pipeline_client = self._build_async_pipeline_client(self.async_auth_policy)

    async def close(self):
        self.pipeline_client.close()
        await self.async_pipeline_client.close()
        await super().close()


class _TableTest(_ServiceTest):
    table_name = "".join(random.choice(_LETTERS) for i in range(30))

    def __init__(self, arguments):
        super().__init__(arguments)
        self.account_endpoint = self.get_from_env("AZURE_STORAGE_TABLES_ENDPOINT")
        self.api_version = "2019-02-02"
        self.data_service_version = "3.0"
        self.sdk_moniker = f"tables/{self.api_version}"
        self._set_auth_policies()

        self.pipeline_client = self._build_sync_pipeline_client(self.sync_auth_policy)
        self.async_pipeline_client = self._build_async_pipeline_client(self.async_auth_policy)

        self.connection_string = self.get_from_env("AZURE_STORAGE_CONN_STR")
        self.async_table_client = TableClient.from_connection_string(self.connection_string, self.table_name)

    async def global_setup(self):
        await super().global_setup()
        await self.async_table_client.create_table()

    async def global_cleanup(self):
        await self.async_table_client.delete_table()

    def get_base_entity(self, pk, rk, size):
        # 227 is the length of the entity with Data of length 0
        base_entity_length = 227
        data_length = max(size - base_entity_length, 0)
        # size = 227 + data_length
        return {
            "PartitionKey": pk,
            "RowKey": rk,
            "Data": "a" * data_length,
        }

    def get_entity(self, rk=0):
        return {"PartitionKey": "pk", "RowKey": str(rk), "Property1": f"a{rk}", "Property2": f"b{rk}"}

    async def close(self):
        self.pipeline_client.close()
        await self.async_pipeline_client.close()
        await super().close()
